# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .roles import (
    Roles,
    AsyncRoles,
    RolesWithRawResponse,
    AsyncRolesWithRawResponse,
    RolesWithStreamingResponse,
    AsyncRolesWithStreamingResponse,
)
from .usage import (
    Usage,
    AsyncUsage,
    UsageWithRawResponse,
    AsyncUsageWithRawResponse,
    UsageWithStreamingResponse,
    AsyncUsageWithStreamingResponse,
)
from .invites import (
    Invites,
    AsyncInvites,
    InvitesWithRawResponse,
    AsyncInvitesWithRawResponse,
    InvitesWithStreamingResponse,
    AsyncInvitesWithStreamingResponse,
)
from ...._compat import cached_property
from .audit_logs import (
    AuditLogs,
    AsyncAuditLogs,
    AuditLogsWithRawResponse,
    AsyncAuditLogsWithRawResponse,
    AuditLogsWithStreamingResponse,
    AsyncAuditLogsWithStreamingResponse,
)
from .spend_limit import (
    SpendLimit,
    AsyncSpendLimit,
    SpendLimitWithRawResponse,
    AsyncSpendLimitWithRawResponse,
    SpendLimitWithStreamingResponse,
    AsyncSpendLimitWithStreamingResponse,
)
from .users.users import (
    Users,
    AsyncUsers,
    UsersWithRawResponse,
    AsyncUsersWithRawResponse,
    UsersWithStreamingResponse,
    AsyncUsersWithStreamingResponse,
)
from ...._resource import SyncAPIResource, AsyncAPIResource
from .certificates import (
    Certificates,
    AsyncCertificates,
    CertificatesWithRawResponse,
    AsyncCertificatesWithRawResponse,
    CertificatesWithStreamingResponse,
    AsyncCertificatesWithStreamingResponse,
)
from .spend_alerts import (
    SpendAlerts,
    AsyncSpendAlerts,
    SpendAlertsWithRawResponse,
    AsyncSpendAlertsWithRawResponse,
    SpendAlertsWithStreamingResponse,
    AsyncSpendAlertsWithStreamingResponse,
)
from .groups.groups import (
    Groups,
    AsyncGroups,
    GroupsWithRawResponse,
    AsyncGroupsWithRawResponse,
    GroupsWithStreamingResponse,
    AsyncGroupsWithStreamingResponse,
)
from .admin_api_keys import (
    AdminAPIKeys,
    AsyncAdminAPIKeys,
    AdminAPIKeysWithRawResponse,
    AsyncAdminAPIKeysWithRawResponse,
    AdminAPIKeysWithStreamingResponse,
    AsyncAdminAPIKeysWithStreamingResponse,
)
from .data_retention import (
    DataRetention,
    AsyncDataRetention,
    DataRetentionWithRawResponse,
    AsyncDataRetentionWithRawResponse,
    DataRetentionWithStreamingResponse,
    AsyncDataRetentionWithStreamingResponse,
)
from .projects.projects import (
    Projects,
    AsyncProjects,
    ProjectsWithRawResponse,
    AsyncProjectsWithRawResponse,
    ProjectsWithStreamingResponse,
    AsyncProjectsWithStreamingResponse,
)

__all__ = ["Organization", "AsyncOrganization"]


class Organization(SyncAPIResource):
    @cached_property
    def audit_logs(self) -> AuditLogs:
        """List user actions and configuration changes within this organization."""
        return AuditLogs(self._client)

    @cached_property
    def admin_api_keys(self) -> AdminAPIKeys:
        return AdminAPIKeys(self._client)

    @cached_property
    def usage(self) -> Usage:
        return Usage(self._client)

    @cached_property
    def invites(self) -> Invites:
        return Invites(self._client)

    @cached_property
    def users(self) -> Users:
        return Users(self._client)

    @cached_property
    def groups(self) -> Groups:
        return Groups(self._client)

    @cached_property
    def roles(self) -> Roles:
        return Roles(self._client)

    @cached_property
    def data_retention(self) -> DataRetention:
        return DataRetention(self._client)

    @cached_property
    def spend_limit(self) -> SpendLimit:
        return SpendLimit(self._client)

    @cached_property
    def spend_alerts(self) -> SpendAlerts:
        return SpendAlerts(self._client)

    @cached_property
    def certificates(self) -> Certificates:
        return Certificates(self._client)

    @cached_property
    def projects(self) -> Projects:
        return Projects(self._client)

    @cached_property
    def with_raw_response(self) -> OrganizationWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return OrganizationWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OrganizationWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return OrganizationWithStreamingResponse(self)


class AsyncOrganization(AsyncAPIResource):
    @cached_property
    def audit_logs(self) -> AsyncAuditLogs:
        """List user actions and configuration changes within this organization."""
        return AsyncAuditLogs(self._client)

    @cached_property
    def admin_api_keys(self) -> AsyncAdminAPIKeys:
        return AsyncAdminAPIKeys(self._client)

    @cached_property
    def usage(self) -> AsyncUsage:
        return AsyncUsage(self._client)

    @cached_property
    def invites(self) -> AsyncInvites:
        return AsyncInvites(self._client)

    @cached_property
    def users(self) -> AsyncUsers:
        return AsyncUsers(self._client)

    @cached_property
    def groups(self) -> AsyncGroups:
        return AsyncGroups(self._client)

    @cached_property
    def roles(self) -> AsyncRoles:
        return AsyncRoles(self._client)

    @cached_property
    def data_retention(self) -> AsyncDataRetention:
        return AsyncDataRetention(self._client)

    @cached_property
    def spend_limit(self) -> AsyncSpendLimit:
        return AsyncSpendLimit(self._client)

    @cached_property
    def spend_alerts(self) -> AsyncSpendAlerts:
        return AsyncSpendAlerts(self._client)

    @cached_property
    def certificates(self) -> AsyncCertificates:
        return AsyncCertificates(self._client)

    @cached_property
    def projects(self) -> AsyncProjects:
        return AsyncProjects(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncOrganizationWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/openai/openai-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOrganizationWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOrganizationWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/openai/openai-python#with_streaming_response
        """
        return AsyncOrganizationWithStreamingResponse(self)


class OrganizationWithRawResponse:
    def __init__(self, organization: Organization) -> None:
        self._organization = organization

    @cached_property
    def audit_logs(self) -> AuditLogsWithRawResponse:
        """List user actions and configuration changes within this organization."""
        return AuditLogsWithRawResponse(self._organization.audit_logs)

    @cached_property
    def admin_api_keys(self) -> AdminAPIKeysWithRawResponse:
        return AdminAPIKeysWithRawResponse(self._organization.admin_api_keys)

    @cached_property
    def usage(self) -> UsageWithRawResponse:
        return UsageWithRawResponse(self._organization.usage)

    @cached_property
    def invites(self) -> InvitesWithRawResponse:
        return InvitesWithRawResponse(self._organization.invites)

    @cached_property
    def users(self) -> UsersWithRawResponse:
        return UsersWithRawResponse(self._organization.users)

    @cached_property
    def groups(self) -> GroupsWithRawResponse:
        return GroupsWithRawResponse(self._organization.groups)

    @cached_property
    def roles(self) -> RolesWithRawResponse:
        return RolesWithRawResponse(self._organization.roles)

    @cached_property
    def data_retention(self) -> DataRetentionWithRawResponse:
        return DataRetentionWithRawResponse(self._organization.data_retention)

    @cached_property
    def spend_limit(self) -> SpendLimitWithRawResponse:
        return SpendLimitWithRawResponse(self._organization.spend_limit)

    @cached_property
    def spend_alerts(self) -> SpendAlertsWithRawResponse:
        return SpendAlertsWithRawResponse(self._organization.spend_alerts)

    @cached_property
    def certificates(self) -> CertificatesWithRawResponse:
        return CertificatesWithRawResponse(self._organization.certificates)

    @cached_property
    def projects(self) -> ProjectsWithRawResponse:
        return ProjectsWithRawResponse(self._organization.projects)


class AsyncOrganizationWithRawResponse:
    def __init__(self, organization: AsyncOrganization) -> None:
        self._organization = organization

    @cached_property
    def audit_logs(self) -> AsyncAuditLogsWithRawResponse:
        """List user actions and configuration changes within this organization."""
        return AsyncAuditLogsWithRawResponse(self._organization.audit_logs)

    @cached_property
    def admin_api_keys(self) -> AsyncAdminAPIKeysWithRawResponse:
        return AsyncAdminAPIKeysWithRawResponse(self._organization.admin_api_keys)

    @cached_property
    def usage(self) -> AsyncUsageWithRawResponse:
        return AsyncUsageWithRawResponse(self._organization.usage)

    @cached_property
    def invites(self) -> AsyncInvitesWithRawResponse:
        return AsyncInvitesWithRawResponse(self._organization.invites)

    @cached_property
    def users(self) -> AsyncUsersWithRawResponse:
        return AsyncUsersWithRawResponse(self._organization.users)

    @cached_property
    def groups(self) -> AsyncGroupsWithRawResponse:
        return AsyncGroupsWithRawResponse(self._organization.groups)

    @cached_property
    def roles(self) -> AsyncRolesWithRawResponse:
        return AsyncRolesWithRawResponse(self._organization.roles)

    @cached_property
    def data_retention(self) -> AsyncDataRetentionWithRawResponse:
        return AsyncDataRetentionWithRawResponse(self._organization.data_retention)

    @cached_property
    def spend_limit(self) -> AsyncSpendLimitWithRawResponse:
        return AsyncSpendLimitWithRawResponse(self._organization.spend_limit)

    @cached_property
    def spend_alerts(self) -> AsyncSpendAlertsWithRawResponse:
        return AsyncSpendAlertsWithRawResponse(self._organization.spend_alerts)

    @cached_property
    def certificates(self) -> AsyncCertificatesWithRawResponse:
        return AsyncCertificatesWithRawResponse(self._organization.certificates)

    @cached_property
    def projects(self) -> AsyncProjectsWithRawResponse:
        return AsyncProjectsWithRawResponse(self._organization.projects)


class OrganizationWithStreamingResponse:
    def __init__(self, organization: Organization) -> None:
        self._organization = organization

    @cached_property
    def audit_logs(self) -> AuditLogsWithStreamingResponse:
        """List user actions and configuration changes within this organization."""
        return AuditLogsWithStreamingResponse(self._organization.audit_logs)

    @cached_property
    def admin_api_keys(self) -> AdminAPIKeysWithStreamingResponse:
        return AdminAPIKeysWithStreamingResponse(self._organization.admin_api_keys)

    @cached_property
    def usage(self) -> UsageWithStreamingResponse:
        return UsageWithStreamingResponse(self._organization.usage)

    @cached_property
    def invites(self) -> InvitesWithStreamingResponse:
        return InvitesWithStreamingResponse(self._organization.invites)

    @cached_property
    def users(self) -> UsersWithStreamingResponse:
        return UsersWithStreamingResponse(self._organization.users)

    @cached_property
    def groups(self) -> GroupsWithStreamingResponse:
        return GroupsWithStreamingResponse(self._organization.groups)

    @cached_property
    def roles(self) -> RolesWithStreamingResponse:
        return RolesWithStreamingResponse(self._organization.roles)

    @cached_property
    def data_retention(self) -> DataRetentionWithStreamingResponse:
        return DataRetentionWithStreamingResponse(self._organization.data_retention)

    @cached_property
    def spend_limit(self) -> SpendLimitWithStreamingResponse:
        return SpendLimitWithStreamingResponse(self._organization.spend_limit)

    @cached_property
    def spend_alerts(self) -> SpendAlertsWithStreamingResponse:
        return SpendAlertsWithStreamingResponse(self._organization.spend_alerts)

    @cached_property
    def certificates(self) -> CertificatesWithStreamingResponse:
        return CertificatesWithStreamingResponse(self._organization.certificates)

    @cached_property
    def projects(self) -> ProjectsWithStreamingResponse:
        return ProjectsWithStreamingResponse(self._organization.projects)


class AsyncOrganizationWithStreamingResponse:
    def __init__(self, organization: AsyncOrganization) -> None:
        self._organization = organization

    @cached_property
    def audit_logs(self) -> AsyncAuditLogsWithStreamingResponse:
        """List user actions and configuration changes within this organization."""
        return AsyncAuditLogsWithStreamingResponse(self._organization.audit_logs)

    @cached_property
    def admin_api_keys(self) -> AsyncAdminAPIKeysWithStreamingResponse:
        return AsyncAdminAPIKeysWithStreamingResponse(self._organization.admin_api_keys)

    @cached_property
    def usage(self) -> AsyncUsageWithStreamingResponse:
        return AsyncUsageWithStreamingResponse(self._organization.usage)

    @cached_property
    def invites(self) -> AsyncInvitesWithStreamingResponse:
        return AsyncInvitesWithStreamingResponse(self._organization.invites)

    @cached_property
    def users(self) -> AsyncUsersWithStreamingResponse:
        return AsyncUsersWithStreamingResponse(self._organization.users)

    @cached_property
    def groups(self) -> AsyncGroupsWithStreamingResponse:
        return AsyncGroupsWithStreamingResponse(self._organization.groups)

    @cached_property
    def roles(self) -> AsyncRolesWithStreamingResponse:
        return AsyncRolesWithStreamingResponse(self._organization.roles)

    @cached_property
    def data_retention(self) -> AsyncDataRetentionWithStreamingResponse:
        return AsyncDataRetentionWithStreamingResponse(self._organization.data_retention)

    @cached_property
    def spend_limit(self) -> AsyncSpendLimitWithStreamingResponse:
        return AsyncSpendLimitWithStreamingResponse(self._organization.spend_limit)

    @cached_property
    def spend_alerts(self) -> AsyncSpendAlertsWithStreamingResponse:
        return AsyncSpendAlertsWithStreamingResponse(self._organization.spend_alerts)

    @cached_property
    def certificates(self) -> AsyncCertificatesWithStreamingResponse:
        return AsyncCertificatesWithStreamingResponse(self._organization.certificates)

    @cached_property
    def projects(self) -> AsyncProjectsWithStreamingResponse:
        return AsyncProjectsWithStreamingResponse(self._organization.projects)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["SafetyIdentifierBlockedWebhookEvent", "Data"]


class Data(BaseModel):
    """Event data payload."""

    safety_category: str
    """The safety category that triggered the block, such as `bio` or `cyber`."""

    safety_identifier: str
    """The stable safety identifier associated with the blocked request."""

    model: Optional[str] = None
    """The model used for the blocked request, if available."""

    project_id: Optional[str] = None
    """The project associated with the blocked request, if available."""

    request_id: Optional[str] = None
    """The OpenAI request ID for the blocked request, if available."""


class SafetyIdentifierBlockedWebhookEvent(BaseModel):
    """Sent when a request associated with a safety identifier has been blocked."""

    id: str
    """The unique ID of the event."""

    created_at: int
    """The Unix timestamp (in seconds) of when the request was blocked."""

    data: Data
    """Event data payload."""

    type: Literal["safety_identifier.blocked"]
    """The type of the event. Always `safety_identifier.blocked`."""

    object: Optional[Literal["event"]] = None
    """The object of the event. Always `event`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ComputerUsePreviewTool"]


class ComputerUsePreviewTool(BaseModel):
    """A tool that controls a virtual computer.

    Learn more about the [computer tool](https://platform.openai.com/docs/guides/tools-computer-use).
    """

    display_height: int
    """The height of the computer display."""

    display_width: int
    """The width of the computer display."""

    environment: Literal["windows", "mac", "linux", "ubuntu", "browser"]
    """The type of computer environment to control."""

    type: Literal["computer_use_preview"]
    """The type of the computer use tool. Always `computer_use_preview`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["OrganizationSpendAlertDeleted"]


class OrganizationSpendAlertDeleted(BaseModel):
    """Confirmation payload returned after deleting an organization spend alert."""

    id: str
    """The deleted spend alert ID."""

    deleted: bool
    """Whether the spend alert was deleted."""

    object: Literal["organization.spend_alert.deleted"]
    """Always `organization.spend_alert.deleted`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .roles import (
    Roles,
    AsyncRoles,
    RolesWithRawResponse,
    AsyncRolesWithRawResponse,
    RolesWithStreamingResponse,
    AsyncRolesWithStreamingResponse,
)
from .users import (
    Users,
    AsyncUsers,
    UsersWithRawResponse,
    AsyncUsersWithRawResponse,
    UsersWithStreamingResponse,
    AsyncUsersWithStreamingResponse,
)

__all__ = [
    "Roles",
    "AsyncRoles",
    "RolesWithRawResponse",
    "AsyncRolesWithRawResponse",
    "RolesWithStreamingResponse",
    "AsyncRolesWithStreamingResponse",
    "Users",
    "AsyncUsers",
    "UsersWithRawResponse",
    "AsyncUsersWithRawResponse",
    "UsersWithStreamingResponse",
    "AsyncUsersWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["RealtimeConversationItemFunctionCallOutput"]


class RealtimeConversationItemFunctionCallOutput(BaseModel):
    """A function call output item in a Realtime conversation."""

    call_id: str
    """The ID of the function call this output is for."""

    output: str
    """
    The output of the function call, this is free text and can contain any
    information or simply be empty.
    """

    type: Literal["function_call_output"]
    """The type of the item. Always `function_call_output`."""

    id: Optional[str] = None
    """The unique ID of the item.

    This may be provided by the client or generated by the server.
    """

    object: Optional[Literal["realtime.item"]] = None
    """Identifier for the API object being returned - always `realtime.item`.

    Optional when creating a new item.
    """

    status: Optional[Literal["completed", "incomplete", "in_progress"]] = None
    """The status of the item. Has no effect on the conversation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["BetaResponseImageGenCallInProgressEvent", "Agent"]


class Agent(BaseModel):
    """The agent that owns this multi-agent streaming event."""

    agent_name: str
    """The canonical name of the agent that produced this item."""


class BetaResponseImageGenCallInProgressEvent(BaseModel):
    """Emitted when an image generation tool call is in progress."""

    item_id: str
    """The unique identifier of the image generation item being processed."""

    output_index: int
    """The index of the output item in the response's output array."""

    sequence_number: int
    """The sequence number of the image generation item being processed."""

    type: Literal["response.image_generation_call.in_progress"]
    """The type of the event. Always 'response.image_generation_call.in_progress'."""

    agent: Optional[Agent] = None
    """The agent that owns this multi-agent streaming event."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ResponseFormatTextGrammar"]


class ResponseFormatTextGrammar(BaseModel):
    """
    A custom grammar for the model to follow when generating text.
    Learn more in the [custom grammars guide](https://platform.openai.com/docs/guides/custom-grammars).
    """

    grammar: str
    """The custom grammar for the model to follow."""

    type: Literal["grammar"]
    """The type of response format being defined. Always `grammar`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["ResponseConversationParamParam"]


class ResponseConversationParamParam(TypedDict, total=False):
    """The conversation that this response belongs to."""

    id: Required[str]
    """The unique ID of the conversation."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union
from typing_extensions import TypeAlias

from .beta_response_input_file_param import BetaResponseInputFileParam
from .beta_response_input_text_param import BetaResponseInputTextParam
from .beta_response_input_image_param import BetaResponseInputImageParam

__all__ = ["BetaResponseInputMessageContentListParam", "BetaResponseInputContentParam"]

BetaResponseInputContentParam: TypeAlias = Union[
    BetaResponseInputTextParam, BetaResponseInputImageParam, BetaResponseInputFileParam
]

BetaResponseInputMessageContentListParam: TypeAlias = List[BetaResponseInputContentParam]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Iterable, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from .beta_response_input_file_param import BetaResponseInputFileParam
from .beta_response_input_text_param import BetaResponseInputTextParam
from .beta_response_input_image_param import BetaResponseInputImageParam

__all__ = [
    "BetaResponseCustomToolCallOutputParam",
    "OutputOutputContentList",
    "Agent",
    "Caller",
    "CallerDirect",
    "CallerProgram",
]

OutputOutputContentList: TypeAlias = Union[
    BetaResponseInputTextParam, BetaResponseInputImageParam, BetaResponseInputFileParam
]


class Agent(TypedDict, total=False):
    """The agent that produced this item."""

    agent_name: Required[str]
    """The canonical name of the agent that produced this item."""


class CallerDirect(TypedDict, total=False):
    type: Required[Literal["direct"]]
    """The caller type. Always `direct`."""


class CallerProgram(TypedDict, total=False):
    caller_id: Required[str]
    """The call ID of the program item that produced this tool call."""

    type: Required[Literal["program"]]
    """The caller type. Always `program`."""


Caller: TypeAlias = Union[CallerDirect, CallerProgram]


class BetaResponseCustomToolCallOutputParam(TypedDict, total=False):
    """The output of a custom tool call from your code, being sent back to the model."""

    call_id: Required[str]
    """The call ID, used to map this custom tool call output to a custom tool call."""

    output: Required[Union[str, Iterable[OutputOutputContentList]]]
    """
    The output from the custom tool call generated by your code. Can be a string or
    an list of output content.
    """

    type: Required[Literal["custom_tool_call_output"]]
    """The type of the custom tool call output. Always `custom_tool_call_output`."""

    id: str
    """The unique ID of the custom tool call output in the OpenAI platform."""

    agent: Optional[Agent]
    """The agent that produced this item."""

    caller: Optional[Caller]
    """The execution context that produced this tool call."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .beta_response_input_item import BetaResponseInputItem

__all__ = ["BetaResponseInput"]

BetaResponseInput: TypeAlias = List[BetaResponseInputItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .input_item_list_params import InputItemListParams as InputItemListParams
from .beta_response_item_list import BetaResponseItemList as BetaResponseItemList
from .input_token_count_params import InputTokenCountParams as InputTokenCountParams
from .input_token_count_response import InputTokenCountResponse as InputTokenCountResponse



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["OutputAudioBufferClearEventParam"]


class OutputAudioBufferClearEventParam(TypedDict, total=False):
    """**WebRTC/SIP Only:** Emit to cut off the current audio response.

    This will trigger the server to
    stop generating audio and emit a `output_audio_buffer.cleared` event. This
    event should be preceded by a `response.cancel` client event to stop the
    generation of the current response.
    [Learn more](https://platform.openai.com/docs/guides/realtime-conversations#client-and-server-events-for-audio-in-webrtc).
    """

    type: Required[Literal["output_audio_buffer.clear"]]
    """The event type, must be `output_audio_buffer.clear`."""

    event_id: str
    """The unique ID of the client event used for error handling."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

from ..._types import SequenceNotStr
from ..shared_params.compound_filter import CompoundFilter
from ..shared_params.comparison_filter import ComparisonFilter

__all__ = ["FileSearchToolParam", "Filters", "RankingOptions", "RankingOptionsHybridSearch"]

Filters: TypeAlias = Union[ComparisonFilter, CompoundFilter]


class RankingOptionsHybridSearch(TypedDict, total=False):
    """
    Weights that control how reciprocal rank fusion balances semantic embedding matches versus sparse keyword matches when hybrid search is enabled.
    """

    embedding_weight: Required[float]
    """The weight of the embedding in the reciprocal ranking fusion."""

    text_weight: Required[float]
    """The weight of the text in the reciprocal ranking fusion."""


class RankingOptions(TypedDict, total=False):
    """Ranking options for search."""

    hybrid_search: RankingOptionsHybridSearch
    """
    Weights that control how reciprocal rank fusion balances semantic embedding
    matches versus sparse keyword matches when hybrid search is enabled.
    """

    ranker: Literal["auto", "default-2024-11-15"]
    """The ranker to use for the file search."""

    score_threshold: float
    """The score threshold for the file search, a number between 0 and 1.

    Numbers closer to 1 will attempt to return only the most relevant results, but
    may return fewer results.
    """


class FileSearchToolParam(TypedDict, total=False):
    """A tool that searches for relevant content from uploaded files.

    Learn more about the [file search tool](https://platform.openai.com/docs/guides/tools-file-search).
    """

    type: Required[Literal["file_search"]]
    """The type of the file search tool. Always `file_search`."""

    vector_store_ids: Required[SequenceNotStr[str]]
    """The IDs of the vector stores to search."""

    filters: Optional[Filters]
    """A filter to apply."""

    max_num_results: int
    """The maximum number of results to return.

    This number should be between 1 and 50 inclusive.
    """

    ranking_options: RankingOptions
    """Ranking options for search."""



#!/usr/bin/env -S rye run python

import os
import ssl

from openai import OpenAI, DefaultHttpx2Client

ssl_context = ssl.create_default_context(
    cafile=os.environ.get("OPENAI_MTLS_CA_BUNDLE"),
)
ssl_context.load_cert_chain(
    # Leaf certificate first; if intermediate-chain support is enabled, follow
    # it with all required intermediates.
    certfile=os.environ["OPENAI_MTLS_CERTIFICATE_CHAIN"],
    keyfile=os.environ["OPENAI_MTLS_PRIVATE_KEY"],
    password=os.environ.get("OPENAI_MTLS_PRIVATE_KEY_PASSWORD"),
)

with OpenAI(
    api_key=os.environ["OPENAI_API_KEY"],
    base_url=os.environ.get(
        "OPENAI_BASE_URL",
        "https://mtls.api.openai.com/v1",
    ),
    http_client=DefaultHttpx2Client(
        verify=ssl_context,
        follow_redirects=False,
    ),
) as client:
    print(client.files.list())



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .roles import (
    Roles,
    AsyncRoles,
    RolesWithRawResponse,
    AsyncRolesWithRawResponse,
    RolesWithStreamingResponse,
    AsyncRolesWithStreamingResponse,
)
from .groups import (
    Groups,
    AsyncGroups,
    GroupsWithRawResponse,
    AsyncGroupsWithRawResponse,
    GroupsWithStreamingResponse,
    AsyncGroupsWithStreamingResponse,
)

__all__ = [
    "Roles",
    "AsyncRoles",
    "RolesWithRawResponse",
    "AsyncRolesWithRawResponse",
    "RolesWithStreamingResponse",
    "AsyncRolesWithStreamingResponse",
    "Groups",
    "AsyncGroups",
    "GroupsWithRawResponse",
    "AsyncGroupsWithRawResponse",
    "GroupsWithStreamingResponse",
    "AsyncGroupsWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["ChatSessionStatus"]

ChatSessionStatus: TypeAlias = Literal["active", "expired", "cancelled"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal, TypeAlias

__all__ = ["RealtimeReasoningEffort"]

RealtimeReasoningEffort: TypeAlias = Literal["minimal", "low", "medium", "high", "xhigh"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["ConversationItemRetrieveEvent"]


class ConversationItemRetrieveEvent(BaseModel):
    """
    Send this event when you want to retrieve the server's representation of a specific item in the conversation history. This is useful, for example, to inspect user audio after noise cancellation and VAD.
    The server will respond with a `conversation.item.retrieved` event,
    unless the item does not exist in the conversation history, in which case the
    server will respond with an error.
    """

    item_id: str
    """The ID of the item to retrieve."""

    type: Literal["conversation.item.retrieve"]
    """The event type, must be `conversation.item.retrieve`."""

    event_id: Optional[str] = None
    """Optional client-generated ID used to identify this event."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["BetaResponseFunctionCallArgumentsDoneEvent", "Agent"]


class Agent(BaseModel):
    """The agent that owns this multi-agent streaming event."""

    agent_name: str
    """The canonical name of the agent that produced this item."""


class BetaResponseFunctionCallArgumentsDoneEvent(BaseModel):
    """Emitted when function-call arguments are finalized."""

    arguments: str
    """The function-call arguments."""

    item_id: str
    """The ID of the item."""

    name: str
    """The name of the function that was called."""

    output_index: int
    """The index of the output item."""

    sequence_number: int
    """The sequence number of this event."""

    type: Literal["response.function_call_arguments.done"]

    agent: Optional[Agent] = None
    """The agent that owns this multi-agent streaming event."""



# pyright: reportMissingTypeStubs=false

from __future__ import annotations

import os
import hashlib
from typing import Any, Literal, Mapping, Callable, Protocol, cast
from dataclasses import field, dataclass

from .._exceptions import OpenAIError

AwsCredentialsProvider = Callable[[], object]


class _BotocoreSession(Protocol):
    def get_credentials(self) -> object | None: ...


_AUTHORIZATION = "authorization"
_AWS_SIGNING_HEADERS = (
    _AUTHORIZATION,
    "x-amz-content-sha256",
    "x-amz-date",
    "x-amz-security-token",
)


def _load_botocore() -> tuple[Any, Any, Any, Any]:
    try:
        from botocore.auth import SigV4Auth  # type: ignore[import-untyped]
        from botocore.session import Session  # type: ignore[import-untyped]
        from botocore.awsrequest import AWSRequest  # type: ignore[import-untyped]
        from botocore.credentials import Credentials  # type: ignore[import-untyped]
    except ImportError as exc:
        raise OpenAIError(
            "Bedrock AWS authentication requires optional AWS dependencies. "
            "Install them with `pip install openai[bedrock]` and try again."
        ) from exc

    return SigV4Auth, AWSRequest, Credentials, Session


@dataclass(frozen=True)
class BedrockAwsAuthConfig:
    region: str
    source: Literal["static", "profile", "provider", "default"]
    region_source: Literal["explicit", "environment", "profile"] = "explicit"
    profile: str | None = None
    access_key_id: str | None = field(default=None, repr=False)
    secret_access_key: str | None = field(default=None, repr=False)
    session_token: str | None = field(default=None, repr=False)
    credentials_provider: AwsCredentialsProvider | None = field(default=None, repr=False, compare=False)


class BedrockAwsAuth:
    def __init__(self, config: BedrockAwsAuthConfig, *, session: _BotocoreSession | None = None) -> None:
        sigv4_auth_cls, aws_request_cls, credentials_cls, session_cls = _load_botocore()

        if session is None:
            try:
                session = session_cls(profile=config.profile)
            except Exception as exc:
                raise OpenAIError(
                    "Failed to resolve AWS credentials for Bedrock. Verify your AWS profile, environment variables, "
                    "or runtime identity configuration and try again."
                ) from exc

        assert session is not None
        self.config = config
        self._session = session
        self._credentials_provider = config.credentials_provider
        self._explicit_credentials = (
            credentials_cls(config.access_key_id, config.secret_access_key, config.session_token)
            if config.access_key_id is not None and config.secret_access_key is not None
            else None
        )
        self._aws_request_cls = aws_request_cls
        self._sigv4_auth_cls = sigv4_auth_cls

    @classmethod
    def resolve(
        cls,
        *,
        region: str | None,
        profile: str | None,
        access_key_id: str | None,
        secret_access_key: str | None,
        session_token: str | None,
        credentials_provider: AwsCredentialsProvider | None,
    ) -> BedrockAwsAuth:
        _, _, _, session_cls = _load_botocore()

        try:
            session = session_cls(profile=profile)
            resolved_region, region_source = resolve_aws_region_with_source(region, session=session)
        except OpenAIError:
            raise
        except Exception as exc:
            raise OpenAIError(
                "Failed to resolve AWS credentials for Bedrock. Verify your AWS profile, environment variables, "
                "or runtime identity configuration and try again."
            ) from exc

        source: Literal["static", "profile", "provider", "default"]
        if access_key_id is not None:
            source = "static"
        elif profile is not None:
            source = "profile"
        elif credentials_provider is not None:
            source = "provider"
        else:
            source = "default"

        config = BedrockAwsAuthConfig(
            region=resolved_region,
            source=source,
            region_source=region_source,
            profile=profile,
            access_key_id=access_key_id,
            secret_access_key=secret_access_key,
            session_token=session_token,
            credentials_provider=credentials_provider,
        )
        return cls(config, session=session)

    def sign(self, *, method: str, url: str, headers: Mapping[str, str], body: bytes | None) -> dict[str, str]:
        try:
            credentials = (
                self._credentials_provider()
                if self._credentials_provider is not None
                else self._explicit_credentials or self._session.get_credentials()
            )
            if credentials is None:
                raise OpenAIError(
                    "Could not find credentials for Bedrock. Pass a bearer credential or AWS credentials to "
                    "`bedrock(...)`, "
                    "set `AWS_BEARER_TOKEN_BEDROCK`, or configure the default AWS credential chain."
                )

            get_frozen_credentials = getattr(credentials, "get_frozen_credentials", None)
            if callable(get_frozen_credentials):
                credentials = get_frozen_credentials()

            signed_headers = {
                name: value for name, value in headers.items() if name.lower() not in _AWS_SIGNING_HEADERS
            }
            signed_headers["X-Amz-Content-SHA256"] = hashlib.sha256(body or b"").hexdigest()
            aws_request = self._aws_request_cls(
                method=method,
                url=url,
                data=body,
                headers=signed_headers,
            )
            self._sigv4_auth_cls(credentials, "bedrock-mantle", self.config.region).add_auth(aws_request)
        except OpenAIError:
            raise
        except Exception as exc:
            raise OpenAIError(
                "Failed to resolve AWS credentials for Bedrock. Verify your AWS profile, environment variables, "
                "or runtime identity configuration and try again."
            ) from exc

        return dict(aws_request.headers.items())


def resolve_aws_region_with_source(
    aws_region: str | None, *, session: object | None = None
) -> tuple[str, Literal["explicit", "environment", "profile"]]:
    region = aws_region
    source: Literal["explicit", "environment", "profile"] = "explicit"
    if region is None or not region.strip():
        region = os.environ.get("AWS_REGION") or os.environ.get("AWS_DEFAULT_REGION")
        source = "environment"
    if (region is None or not region.strip()) and session is not None:
        get_config_variable = getattr(session, "get_config_variable", None)
        if callable(get_config_variable):
            region = cast("str | None", get_config_variable("region"))
            source = "profile"

    if region is None or not region.strip():
        raise OpenAIError(
            "Bedrock requires an AWS region. Pass `region` to `bedrock(...)`, or set `AWS_REGION` or "
            "`AWS_DEFAULT_REGION`."
        )

    return region.strip(), source



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["GroupCreateParams"]


class GroupCreateParams(TypedDict, total=False):
    group_id: Required[str]
    """Identifier of the group to add to the project."""

    role: Required[str]
    """Identifier of the project role to grant to the group."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["BetaResponseCodeInterpreterCallInProgressEvent", "Agent"]


class Agent(BaseModel):
    """The agent that owns this multi-agent streaming event."""

    agent_name: str
    """The canonical name of the agent that produced this item."""


class BetaResponseCodeInterpreterCallInProgressEvent(BaseModel):
    """Emitted when a code interpreter call is in progress."""

    item_id: str
    """The unique identifier of the code interpreter tool call item."""

    output_index: int
    """
    The index of the output item in the response for which the code interpreter call
    is in progress.
    """

    sequence_number: int
    """The sequence number of this event, used to order streaming events."""

    type: Literal["response.code_interpreter_call.in_progress"]
    """The type of the event. Always `response.code_interpreter_call.in_progress`."""

    agent: Optional[Agent] = None
    """The agent that owns this multi-agent streaming event."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .._types import FileTypes

__all__ = ["ContentProvenanceCheckCreateParams"]


class ContentProvenanceCheckCreateParams(TypedDict, total=False):
    file: Required[FileTypes]
    """The image or audio file to check for supported OpenAI provenance signals."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["BetaResponseCustomToolCallInputDoneEvent", "Agent"]


class Agent(BaseModel):
    """The agent that owns this multi-agent streaming event."""

    agent_name: str
    """The canonical name of the agent that produced this item."""


class BetaResponseCustomToolCallInputDoneEvent(BaseModel):
    """Event indicating that input for a custom tool call is complete."""

    input: str
    """The complete input data for the custom tool call."""

    item_id: str
    """Unique identifier for the API item associated with this event."""

    output_index: int
    """The index of the output this event applies to."""

    sequence_number: int
    """The sequence number of this event."""

    type: Literal["response.custom_tool_call_input.done"]
    """The event type identifier."""

    agent: Optional[Agent] = None
    """The agent that owns this multi-agent streaming event."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from ....._types import SequenceNotStr

__all__ = ["CertificateActivateParams"]


class CertificateActivateParams(TypedDict, total=False):
    certificate_ids: Required[SequenceNotStr[str]]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["UserCreateResponse"]


class UserCreateResponse(BaseModel):
    """Confirmation payload returned after adding a user to a group."""

    group_id: str
    """Identifier of the group the user was added to."""

    object: Literal["group.user"]
    """Always `group.user`."""

    user_id: str
    """Identifier of the user that was added."""



#!/usr/bin/env -S poetry run python

import asyncio

from openai import AsyncOpenAI

# gets API Key from environment variable OPENAI_API_KEY
client = AsyncOpenAI()


async def main() -> None:
    stream = await client.chat.completions.create(
        model="gpt-5.5",
        messages=[
            {
                "role": "user",
                "content": "Say this is a test",
            },
        ],
        stream=True,
    )
    async for chunk in stream:
        if not chunk.choices:
            continue

        print(chunk.choices[0].delta.content, end="")
    print()


asyncio.run(main())



from __future__ import annotations

import builtins
from typing import Any, Iterator, cast

import httpx
import pytest

import openai.lib._bedrock_auth as bedrock_auth_module
from openai import OpenAI, AsyncOpenAI, OpenAIError
from tests.utils import update_env
from openai._types import Omit
from openai._provider import _create_provider, _ProviderRuntime
from openai.providers import bedrock


def test_sync_provider_owns_endpoint_and_bearer_authentication() -> None:
    requests: list[httpx.Request] = []

    def handler(request: httpx.Request) -> httpx.Response:
        requests.append(request)
        return httpx.Response(200, request=request, json={})

    client = OpenAI(
        provider=bedrock(region="us-east-1", api_key="bedrock token"),
        http_client=httpx.Client(transport=httpx.MockTransport(handler), trust_env=False),
    )
    client.get("/models", cast_to=httpx.Response)

    assert client.base_url == httpx.URL("https://bedrock-mantle.us-east-1.api.aws/openai/v1/")
    assert requests[0].url == httpx.URL("https://bedrock-mantle.us-east-1.api.aws/openai/v1/models")
    assert requests[0].headers["Authorization"] == "Bearer bedrock token"


@pytest.mark.asyncio
async def test_async_provider_owns_endpoint_and_bearer_authentication() -> None:
    requests: list[httpx.Request] = []

    async def handler(request: httpx.Request) -> httpx.Response:
        requests.append(request)
        return httpx.Response(200, request=request, json={})

    client = AsyncOpenAI(
        provider=bedrock(region="us-east-1", token_provider=lambda: "bedrock token"),
        http_client=httpx.AsyncClient(transport=httpx.MockTransport(handler), trust_env=False),
    )
    await client.get("/models", cast_to=httpx.Response)
    await client.close()

    assert requests[0].headers["Authorization"] == "Bearer bedrock token"


def test_provider_ignores_openai_environment_configuration() -> None:
    requests: list[httpx.Request] = []

    def handler(request: httpx.Request) -> httpx.Response:
        requests.append(request)
        return httpx.Response(200, request=request, json={})

    with update_env(
        OPENAI_API_KEY="openai token",
        OPENAI_BASE_URL="https://api.openai.invalid/v1",
        OPENAI_CUSTOM_HEADERS="Authorization: Bearer openai custom token",
    ):
        client = OpenAI(
            provider=bedrock(region="us-east-1", api_key="bedrock token"),
            http_client=httpx.Client(transport=httpx.MockTransport(handler), trust_env=False),
        )
        client.get("/models", cast_to=httpx.Response)

    assert client.api_key == ""
    assert requests[0].url.host == "bedrock-mantle.us-east-1.api.aws"
    assert requests[0].headers["Authorization"] == "Bearer bedrock token"


@pytest.mark.parametrize(
    ("option", "value"),
    [
        ("api_key", "openai token"),
        ("admin_api_key", "admin token"),
        ("workload_identity", cast(Any, object())),
        ("base_url", "https://api.openai.invalid/v1"),
    ],
)
def test_provider_rejects_top_level_authentication_and_routing(option: str, value: object) -> None:
    with pytest.raises(
        OpenAIError,
        match=rf"`provider` cannot be combined with top-level `{option}`.*`bedrock\(\.\.\.\)`",
    ):
        OpenAI(provider=bedrock(region="us-east-1", api_key="bedrock token"), **{option: value})  # type: ignore[arg-type]


def test_provider_survives_with_options_and_can_be_replaced() -> None:
    client = OpenAI(provider=bedrock(region="us-east-1", api_key="first"))

    copied = client.with_options(timeout=1)
    replaced = client.with_options(provider=bedrock(region="eu-west-1", api_key="second"))

    assert copied.base_url == client.base_url
    assert copied._provider is client._provider
    assert replaced.base_url == httpx.URL("https://bedrock-mantle.eu-west-1.api.aws/openai/v1/")
    assert replaced._provider is not client._provider


def test_switching_to_provider_drops_inherited_openai_metadata() -> None:
    requests: list[httpx.Request] = []

    def handler(request: httpx.Request) -> httpx.Response:
        requests.append(request)
        return httpx.Response(200, request=request, json={})

    with update_env(
        OPENAI_CUSTOM_HEADERS="X-OpenAI-Ambient: leak",
        OPENAI_ORG_ID="ambient-org",
        OPENAI_PROJECT_ID="ambient-project",
    ):
        client = OpenAI(
            api_key="openai token",
            default_headers={"X-OpenAI-Custom": "leak"},
            http_client=httpx.Client(transport=httpx.MockTransport(handler), trust_env=False),
        )
        provider_client = client.with_options(provider=bedrock(region="us-east-1", api_key="bedrock token"))

    provider_client.get("/models", cast_to=httpx.Response)

    headers = requests[0].headers
    assert headers["Authorization"] == "Bearer bedrock token"
    assert "X-OpenAI-Ambient" not in headers
    assert "X-OpenAI-Custom" not in headers
    assert "OpenAI-Organization" not in headers
    assert "OpenAI-Project" not in headers


@pytest.mark.asyncio
async def test_async_switching_to_provider_drops_inherited_openai_metadata() -> None:
    requests: list[httpx.Request] = []

    async def handler(request: httpx.Request) -> httpx.Response:
        requests.append(request)
        return httpx.Response(200, request=request, json={})

    client = AsyncOpenAI(
        api_key="openai token",
        organization="openai-org",
        project="openai-project",
        default_headers={"X-OpenAI-Custom": "leak"},
        http_client=httpx.AsyncClient(transport=httpx.MockTransport(handler), trust_env=False),
    )
    provider_client = client.with_options(provider=bedrock(region="us-east-1", api_key="bedrock token"))

    await provider_client.get("/models", cast_to=httpx.Response)
    await provider_client.close()

    headers = requests[0].headers
    assert headers["Authorization"] == "Bearer bedrock token"
    assert "X-OpenAI-Custom" not in headers
    assert "OpenAI-Organization" not in headers
    assert "OpenAI-Project" not in headers


def test_provider_metadata_survives_same_provider_clone_but_not_replacement() -> None:
    requests: list[httpx.Request] = []

    def handler(request: httpx.Request) -> httpx.Response:
        requests.append(request)
        return httpx.Response(200, request=request, json={})

    first_provider = bedrock(region="us-east-1", api_key="first token")
    client = OpenAI(
        provider=first_provider,
        organization="provider-org",
        project="provider-project",
        default_headers={"X-Provider-Custom": "preserve-me"},
        http_client=httpx.Client(transport=httpx.MockTransport(handler), trust_env=False),
    )

    client.with_options(timeout=1).get("/models", cast_to=httpx.Response)
    client.with_options(provider=bedrock(region="us-east-1", api_key="second token")).get(
        "/models", cast_to=httpx.Response
    )

    same_provider_headers, replacement_headers = (request.headers for request in requests)
    assert same_provider_headers["X-Provider-Custom"] == "preserve-me"
    assert same_provider_headers["OpenAI-Organization"] == "provider-org"
    assert same_provider_headers["OpenAI-Project"] == "provider-project"
    assert "X-Provider-Custom" not in replacement_headers
    assert "OpenAI-Organization" not in replacement_headers
    assert "OpenAI-Project" not in replacement_headers


def test_provider_normalizes_responses_before_status_handling() -> None:
    class NormalizingProvider:
        name = "normalizing"

        def configure(self) -> _ProviderRuntime:
            def normalize(response: httpx.Response) -> httpx.Response:
                return httpx.Response(200, request=response.request, json={"normalized": True})

            return _ProviderRuntime(
                name=self.name,
                base_url="https://provider.example/v1",
                normalize_response=normalize,
            )

    client = OpenAI(
        provider=_create_provider(NormalizingProvider()),
        max_retries=0,
        http_client=httpx.Client(
            transport=httpx.MockTransport(lambda request: httpx.Response(500, request=request, json={})),
            trust_env=False,
        ),
    )

    response = client.get("/models", cast_to=httpx.Response)

    assert response.status_code == 200
    assert response.json() == {"normalized": True}


def test_environment_bearer_mode_survives_clone_and_refreshes_each_attempt() -> None:
    requests: list[httpx.Request] = []

    def handler(request: httpx.Request) -> httpx.Response:
        requests.append(request)
        return httpx.Response(200, request=request, json={})

    with update_env(AWS_BEARER_TOKEN_BEDROCK="first token"):
        client = OpenAI(
            provider=bedrock(region="us-east-1"),
            http_client=httpx.Client(transport=httpx.MockTransport(handler), trust_env=False),
        )
        client.get("/models", cast_to=httpx.Response)

    copied = client.with_options(timeout=1)
    with update_env(AWS_BEARER_TOKEN_BEDROCK="second token"):
        copied.get("/models", cast_to=httpx.Response)

    assert [request.headers["Authorization"] for request in requests] == ["Bearer first token", "Bearer second token"]


def test_provider_can_be_removed_with_explicit_openai_credentials() -> None:
    with update_env(OPENAI_CUSTOM_HEADERS=Omit(), OPENAI_ORG_ID=Omit(), OPENAI_PROJECT_ID=Omit()):
        client = OpenAI(
            provider=bedrock(region="us-east-1", api_key="bedrock token"),
            organization="provider-org",
            project="provider-project",
            default_headers={"X-Provider-Custom": "provider value"},
        )

        copied = client.with_options(provider=None, api_key="openai token")

    assert copied._provider is None
    assert copied.api_key == "openai token"
    assert copied.base_url == httpx.URL("https://api.openai.com/v1/")
    assert copied.organization is None
    assert copied.project is None
    assert "X-Provider-Custom" not in copied.default_headers


def test_bearer_provider_does_not_load_botocore(monkeypatch: pytest.MonkeyPatch) -> None:
    def load_botocore() -> None:
        raise AssertionError("bearer authentication must not import botocore")

    monkeypatch.setattr(bedrock_auth_module, "_load_botocore", load_botocore)

    client = OpenAI(provider=bedrock(region="us-east-1", api_key="bedrock token"))
    request = client._build_request(client._prepare_options(_get_options()))
    client._prepare_request(request)

    assert request.headers["Authorization"] == "Bearer bedrock token"


def test_missing_aws_dependency_is_actionable_and_lazy(monkeypatch: pytest.MonkeyPatch) -> None:
    real_import = builtins.__import__
    network_calls = 0

    def import_module(
        name: str,
        globals: dict[str, Any] | None = None,
        locals: dict[str, Any] | None = None,
        fromlist: tuple[str, ...] = (),
        level: int = 0,
    ) -> Any:
        if name.startswith("botocore"):
            raise ImportError(name)
        return real_import(name, globals, locals, fromlist, level)

    def handler(request: httpx.Request) -> httpx.Response:
        nonlocal network_calls
        network_calls += 1
        return httpx.Response(200, request=request)

    monkeypatch.setattr(builtins, "__import__", import_module)
    client = OpenAI(
        provider=bedrock(region="us-east-1"),
        http_client=httpx.Client(transport=httpx.MockTransport(handler), trust_env=False),
    )

    with pytest.raises(OpenAIError, match=r"pip install openai\[bedrock\]"):
        client.get("/models", cast_to=httpx.Response)

    assert network_calls == 0


def test_api_key_none_skips_environment_bearer_fallback() -> None:
    requests: list[httpx.Request] = []

    def handler(request: httpx.Request) -> httpx.Response:
        requests.append(request)
        return httpx.Response(200, request=request, json={})

    with update_env(
        AWS_BEARER_TOKEN_BEDROCK="environment bearer",
        AWS_ACCESS_KEY_ID="access key",
        AWS_SECRET_ACCESS_KEY="secret key",
    ):
        client = OpenAI(
            provider=bedrock(region="us-east-1", api_key=None),
            http_client=httpx.Client(transport=httpx.MockTransport(handler), trust_env=False),
        )
        client.get("/models", cast_to=httpx.Response)

    assert requests[0].headers["Authorization"].startswith("AWS4-HMAC-SHA256 Credential=access key/")


def test_provider_rejects_custom_authorization_before_network() -> None:
    network_calls = 0

    def handler(request: httpx.Request) -> httpx.Response:
        nonlocal network_calls
        network_calls += 1
        return httpx.Response(200, request=request)

    client = OpenAI(
        provider=bedrock(region="us-east-1", api_key="bedrock token"),
        http_client=httpx.Client(transport=httpx.MockTransport(handler), trust_env=False),
    )

    with pytest.raises(OpenAIError, match="cannot be combined with a custom `Authorization` header"):
        client.get(
            "/models",
            cast_to=httpx.Response,
            options={"headers": {"Authorization": "Bearer custom"}},
        )

    assert network_calls == 0


def test_bearer_provider_rejects_cross_origin_requests_before_resolving_credentials() -> None:
    network_calls = 0
    provider_calls = 0

    def token_provider() -> str:
        nonlocal provider_calls
        provider_calls += 1
        return "bedrock token"

    def handler(request: httpx.Request) -> httpx.Response:
        nonlocal network_calls
        network_calls += 1
        return httpx.Response(200, request=request)

    client = OpenAI(
        provider=bedrock(base_url="https://bedrock.example/openai/v1", token_provider=token_provider),
        http_client=httpx.Client(transport=httpx.MockTransport(handler), trust_env=False),
    )

    with pytest.raises(OpenAIError, match="origin other than the configured provider URL"):
        client.get("https://attacker.example/steal", cast_to=httpx.Response)

    assert (provider_calls, network_calls) == (0, 0)


@pytest.mark.asyncio
async def test_async_bearer_provider_rejects_cross_origin_requests_before_resolving_credentials() -> None:
    network_calls = 0
    provider_calls = 0

    async def token_provider() -> str:
        nonlocal provider_calls
        provider_calls += 1
        return "bedrock token"

    async def handler(request: httpx.Request) -> httpx.Response:
        nonlocal network_calls
        network_calls += 1
        return httpx.Response(200, request=request)

    client = AsyncOpenAI(
        provider=bedrock(base_url="https://bedrock.example/openai/v1", token_provider=token_provider),
        http_client=httpx.AsyncClient(transport=httpx.MockTransport(handler), trust_env=False),
    )

    with pytest.raises(OpenAIError, match="origin other than the configured provider URL"):
        await client.get("https://attacker.example/steal", cast_to=httpx.Response)

    await client.close()
    assert (provider_calls, network_calls) == (0, 0)


def test_bearer_provider_allows_one_shot_body_when_retries_are_disabled() -> None:
    requests: list[httpx.Request] = []

    def body() -> Iterator[bytes]:
        yield b"body"

    def handler(request: httpx.Request) -> httpx.Response:
        requests.append(request)
        return httpx.Response(200, request=request)

    client = OpenAI(
        provider=bedrock(base_url="https://bedrock.example/openai/v1", api_key="bedrock token"),
        max_retries=0,
        http_client=httpx.Client(transport=httpx.MockTransport(handler), trust_env=False),
    )

    client.post("/responses", content=body(), cast_to=httpx.Response)

    assert requests[0].content == b"body"


def test_opaque_provider_repr_does_not_expose_credentials() -> None:
    provider = bedrock(
        region="us-east-1",
        access_key_id="secret access key id",
        secret_access_key="secret access key",
        session_token="secret session token",
    )

    assert "secret" not in repr(provider)


def _get_options() -> Any:
    from openai._models import FinalRequestOptions

    return FinalRequestOptions(method="get", url="/models", security={"bearer_auth": True})



#!/usr/bin/env -S poetry run python

import asyncio

from openai import AsyncOpenAI

client = AsyncOpenAI()


async def main() -> None:
    video = await client.videos.create_and_poll(
        model="sora-2",
        prompt="A video of the words 'Thank you' in sparkling letters",
    )

    if video.status == "completed":
        print("Video successfully completed: ", video)
    else:
        print("Video creation failed. Status: ", video.status)


asyncio.run(main())



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["InputAudioBufferClearEvent"]


class InputAudioBufferClearEvent(BaseModel):
    """Send this event to clear the audio bytes in the buffer.

    The server will
    respond with an `input_audio_buffer.cleared` event.
    """

    type: Literal["input_audio_buffer.clear"]
    """The event type, must be `input_audio_buffer.clear`."""

    event_id: Optional[str] = None
    """Optional client-generated ID used to identify this event."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..._models import BaseModel

__all__ = ["LocalSkill"]


class LocalSkill(BaseModel):
    description: str
    """The description of the skill."""

    name: str
    """The name of the skill."""

    path: str
    """The path to the directory containing the skill."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["APIKeyDeleteResponse"]


class APIKeyDeleteResponse(BaseModel):
    id: str

    deleted: bool

    object: Literal["organization.project.api_key.deleted"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .thread import Thread as Thread
from .assistant import Assistant as Assistant
from .beta_tool import BetaTool as BetaTool
from .beta_response import BetaResponse as BetaResponse
from .function_tool import FunctionTool as FunctionTool
from .assistant_tool import AssistantTool as AssistantTool
from .thread_deleted import ThreadDeleted as ThreadDeleted
from .beta_tool_param import BetaToolParam as BetaToolParam
from .beta_custom_tool import BetaCustomTool as BetaCustomTool
from .beta_local_skill import BetaLocalSkill as BetaLocalSkill
from .chatkit_workflow import ChatKitWorkflow as ChatKitWorkflow
from .file_search_tool import FileSearchTool as FileSearchTool
from .assistant_deleted import AssistantDeleted as AssistantDeleted
from .beta_inline_skill import BetaInlineSkill as BetaInlineSkill
from .beta_computer_tool import BetaComputerTool as BetaComputerTool
from .beta_function_tool import BetaFunctionTool as BetaFunctionTool
from .beta_response_item import BetaResponseItem as BetaResponseItem
from .beta_container_auto import BetaContainerAuto as BetaContainerAuto
from .beta_namespace_tool import BetaNamespaceTool as BetaNamespaceTool
from .beta_response_error import BetaResponseError as BetaResponseError
from .beta_response_input import BetaResponseInput as BetaResponseInput
from .beta_response_usage import BetaResponseUsage as BetaResponseUsage
from .function_tool_param import FunctionToolParam as FunctionToolParam
from .assistant_tool_param import AssistantToolParam as AssistantToolParam
from .beta_computer_action import BetaComputerAction as BetaComputerAction
from .beta_response_prompt import BetaResponsePrompt as BetaResponsePrompt
from .beta_response_status import BetaResponseStatus as BetaResponseStatus
from .beta_skill_reference import BetaSkillReference as BetaSkillReference
from .beta_tool_choice_mcp import BetaToolChoiceMcp as BetaToolChoiceMcp
from .beta_web_search_tool import BetaWebSearchTool as BetaWebSearchTool
from .thread_create_params import ThreadCreateParams as ThreadCreateParams
from .thread_update_params import ThreadUpdateParams as ThreadUpdateParams
from .assistant_list_params import AssistantListParams as AssistantListParams
from .assistant_tool_choice import AssistantToolChoice as AssistantToolChoice
from .beta_apply_patch_tool import BetaApplyPatchTool as BetaApplyPatchTool
from .beta_file_search_tool import BetaFileSearchTool as BetaFileSearchTool
from .beta_tool_search_tool import BetaToolSearchTool as BetaToolSearchTool
from .code_interpreter_tool import CodeInterpreterTool as CodeInterpreterTool
from .assistant_stream_event import AssistantStreamEvent as AssistantStreamEvent
from .beta_custom_tool_param import BetaCustomToolParam as BetaCustomToolParam
from .beta_local_environment import BetaLocalEnvironment as BetaLocalEnvironment
from .beta_local_skill_param import BetaLocalSkillParam as BetaLocalSkillParam
from .beta_tool_choice_shell import BetaToolChoiceShell as BetaToolChoiceShell
from .beta_tool_choice_types import BetaToolChoiceTypes as BetaToolChoiceTypes
from .file_search_tool_param import FileSearchToolParam as FileSearchToolParam
from .response_create_params import ResponseCreateParams as ResponseCreateParams
from .assistant_create_params import AssistantCreateParams as AssistantCreateParams
from .assistant_update_params import AssistantUpdateParams as AssistantUpdateParams
from .beta_compacted_response import BetaCompactedResponse as BetaCompactedResponse
from .beta_easy_input_message import BetaEasyInputMessage as BetaEasyInputMessage
from .beta_inline_skill_param import BetaInlineSkillParam as BetaInlineSkillParam
from .beta_tool_choice_custom import BetaToolChoiceCustom as BetaToolChoiceCustom
from .response_compact_params import ResponseCompactParams as ResponseCompactParams
from .beta_computer_tool_param import BetaComputerToolParam as BetaComputerToolParam
from .beta_container_reference import BetaContainerReference as BetaContainerReference
from .beta_function_shell_tool import BetaFunctionShellTool as BetaFunctionShellTool
from .beta_function_tool_param import BetaFunctionToolParam as BetaFunctionToolParam
from .beta_inline_skill_source import BetaInlineSkillSource as BetaInlineSkillSource
from .beta_response_includable import BetaResponseIncludable as BetaResponseIncludable
from .beta_response_input_file import BetaResponseInputFile as BetaResponseInputFile
from .beta_response_input_item import BetaResponseInputItem as BetaResponseInputItem
from .beta_response_input_text import BetaResponseInputText as BetaResponseInputText
from .beta_tool_choice_allowed import BetaToolChoiceAllowed as BetaToolChoiceAllowed
from .beta_tool_choice_options import BetaToolChoiceOptions as BetaToolChoiceOptions
from .response_retrieve_params import ResponseRetrieveParams as ResponseRetrieveParams
from .beta_computer_action_list import BetaComputerActionList as BetaComputerActionList
from .beta_container_auto_param import BetaContainerAutoParam as BetaContainerAutoParam
from .beta_namespace_tool_param import BetaNamespaceToolParam as BetaNamespaceToolParam
from .beta_response_error_event import BetaResponseErrorEvent as BetaResponseErrorEvent
from .beta_response_input_image import BetaResponseInputImage as BetaResponseInputImage
from .beta_response_input_param import BetaResponseInputParam as BetaResponseInputParam
from .beta_response_output_item import BetaResponseOutputItem as BetaResponseOutputItem
from .beta_response_output_text import BetaResponseOutputText as BetaResponseOutputText
from .beta_response_text_config import BetaResponseTextConfig as BetaResponseTextConfig
from .beta_tool_choice_function import BetaToolChoiceFunction as BetaToolChoiceFunction
from .beta_computer_action_param import BetaComputerActionParam as BetaComputerActionParam
from .beta_response_failed_event import BetaResponseFailedEvent as BetaResponseFailedEvent
from .beta_response_inject_event import BetaResponseInjectEvent as BetaResponseInjectEvent
from .beta_response_prompt_param import BetaResponsePromptParam as BetaResponsePromptParam
from .beta_response_queued_event import BetaResponseQueuedEvent as BetaResponseQueuedEvent
from .beta_response_stream_event import BetaResponseStreamEvent as BetaResponseStreamEvent
from .beta_skill_reference_param import BetaSkillReferenceParam as BetaSkillReferenceParam
from .beta_tool_choice_mcp_param import BetaToolChoiceMcpParam as BetaToolChoiceMcpParam
from .beta_web_search_tool_param import BetaWebSearchToolParam as BetaWebSearchToolParam
from .assistant_tool_choice_param import AssistantToolChoiceParam as AssistantToolChoiceParam
from .beta_apply_patch_tool_param import BetaApplyPatchToolParam as BetaApplyPatchToolParam
from .beta_file_search_tool_param import BetaFileSearchToolParam as BetaFileSearchToolParam
from .beta_response_created_event import BetaResponseCreatedEvent as BetaResponseCreatedEvent
from .beta_response_input_content import BetaResponseInputContent as BetaResponseInputContent
from .beta_responses_client_event import BetaResponsesClientEvent as BetaResponsesClientEvent
from .beta_responses_server_event import BetaResponsesServerEvent as BetaResponsesServerEvent
from .beta_tool_search_tool_param import BetaToolSearchToolParam as BetaToolSearchToolParam
from .code_interpreter_tool_param import CodeInterpreterToolParam as CodeInterpreterToolParam
from .assistant_tool_choice_option import AssistantToolChoiceOption as AssistantToolChoiceOption
from .beta_local_environment_param import BetaLocalEnvironmentParam as BetaLocalEnvironmentParam
from .beta_response_output_message import BetaResponseOutputMessage as BetaResponseOutputMessage
from .beta_response_output_refusal import BetaResponseOutputRefusal as BetaResponseOutputRefusal
from .beta_response_reasoning_item import BetaResponseReasoningItem as BetaResponseReasoningItem
from .beta_tool_choice_apply_patch import BetaToolChoiceApplyPatch as BetaToolChoiceApplyPatch
from .beta_tool_choice_shell_param import BetaToolChoiceShellParam as BetaToolChoiceShellParam
from .beta_tool_choice_types_param import BetaToolChoiceTypesParam as BetaToolChoiceTypesParam
from .beta_web_search_preview_tool import BetaWebSearchPreviewTool as BetaWebSearchPreviewTool
from .thread_create_and_run_params import ThreadCreateAndRunParams as ThreadCreateAndRunParams
from .beta_easy_input_message_param import BetaEasyInputMessageParam as BetaEasyInputMessageParam
from .beta_response_compaction_item import BetaResponseCompactionItem as BetaResponseCompactionItem
from .beta_response_completed_event import BetaResponseCompletedEvent as BetaResponseCompletedEvent
from .beta_response_text_done_event import BetaResponseTextDoneEvent as BetaResponseTextDoneEvent
from .beta_tool_choice_custom_param import BetaToolChoiceCustomParam as BetaToolChoiceCustomParam
from .assistant_tool_choice_function import AssistantToolChoiceFunction as AssistantToolChoiceFunction
from .beta_computer_use_preview_tool import BetaComputerUsePreviewTool as BetaComputerUsePreviewTool
from .beta_container_reference_param import BetaContainerReferenceParam as BetaContainerReferenceParam
from .beta_function_shell_tool_param import BetaFunctionShellToolParam as BetaFunctionShellToolParam
from .beta_inline_skill_source_param import BetaInlineSkillSourceParam as BetaInlineSkillSourceParam
from .beta_response_audio_done_event import BetaResponseAudioDoneEvent as BetaResponseAudioDoneEvent
from .beta_response_custom_tool_call import BetaResponseCustomToolCall as BetaResponseCustomToolCall
from .beta_response_incomplete_event import BetaResponseIncompleteEvent as BetaResponseIncompleteEvent
from .beta_response_input_file_param import BetaResponseInputFileParam as BetaResponseInputFileParam
from .beta_response_input_item_param import BetaResponseInputItemParam as BetaResponseInputItemParam
from .beta_response_input_text_param import BetaResponseInputTextParam as BetaResponseInputTextParam
from .beta_response_text_delta_event import BetaResponseTextDeltaEvent as BetaResponseTextDeltaEvent
from .beta_response_tool_search_call import BetaResponseToolSearchCall as BetaResponseToolSearchCall
from .beta_tool_choice_allowed_param import BetaToolChoiceAllowedParam as BetaToolChoiceAllowedParam
from .beta_computer_action_list_param import BetaComputerActionListParam as BetaComputerActionListParam
from .beta_response_audio_delta_event import BetaResponseAudioDeltaEvent as BetaResponseAudioDeltaEvent
from .beta_response_in_progress_event import BetaResponseInProgressEvent as BetaResponseInProgressEvent
from .beta_response_input_image_param import BetaResponseInputImageParam as BetaResponseInputImageParam
from .beta_response_local_environment import BetaResponseLocalEnvironment as BetaResponseLocalEnvironment
from .beta_response_output_text_param import BetaResponseOutputTextParam as BetaResponseOutputTextParam
from .beta_response_text_config_param import BetaResponseTextConfigParam as BetaResponseTextConfigParam
from .beta_tool_choice_function_param import BetaToolChoiceFunctionParam as BetaToolChoiceFunctionParam
from .assistant_response_format_option import AssistantResponseFormatOption as AssistantResponseFormatOption
from .beta_response_computer_tool_call import BetaResponseComputerToolCall as BetaResponseComputerToolCall
from .beta_response_conversation_param import BetaResponseConversationParam as BetaResponseConversationParam
from .beta_response_format_text_config import BetaResponseFormatTextConfig as BetaResponseFormatTextConfig
from .beta_response_function_tool_call import BetaResponseFunctionToolCall as BetaResponseFunctionToolCall
from .beta_response_inject_event_param import BetaResponseInjectEventParam as BetaResponseInjectEventParam
from .beta_response_input_file_content import BetaResponseInputFileContent as BetaResponseInputFileContent
from .beta_response_input_message_item import BetaResponseInputMessageItem as BetaResponseInputMessageItem
from .beta_response_input_text_content import BetaResponseInputTextContent as BetaResponseInputTextContent
from .beta_response_refusal_done_event import BetaResponseRefusalDoneEvent as BetaResponseRefusalDoneEvent
from .beta_response_container_reference import BetaResponseContainerReference as BetaResponseContainerReference
from .beta_response_function_web_search import BetaResponseFunctionWebSearch as BetaResponseFunctionWebSearch
from .beta_response_inject_failed_event import BetaResponseInjectFailedEvent as BetaResponseInjectFailedEvent
from .beta_response_input_content_param import BetaResponseInputContentParam as BetaResponseInputContentParam
from .beta_response_input_image_content import BetaResponseInputImageContent as BetaResponseInputImageContent
from .beta_response_refusal_delta_event import BetaResponseRefusalDeltaEvent as BetaResponseRefusalDeltaEvent
from .beta_responses_client_event_param import BetaResponsesClientEventParam as BetaResponsesClientEventParam
from .assistant_tool_choice_option_param import AssistantToolChoiceOptionParam as AssistantToolChoiceOptionParam
from .beta_response_inject_created_event import BetaResponseInjectCreatedEvent as BetaResponseInjectCreatedEvent
from .beta_response_output_message_param import BetaResponseOutputMessageParam as BetaResponseOutputMessageParam
from .beta_response_output_refusal_param import BetaResponseOutputRefusalParam as BetaResponseOutputRefusalParam
from .beta_response_reasoning_item_param import BetaResponseReasoningItemParam as BetaResponseReasoningItemParam
from .beta_tool_choice_apply_patch_param import BetaToolChoiceApplyPatchParam as BetaToolChoiceApplyPatchParam
from .beta_web_search_preview_tool_param import BetaWebSearchPreviewToolParam as BetaWebSearchPreviewToolParam
from .beta_response_apply_patch_tool_call import BetaResponseApplyPatchToolCall as BetaResponseApplyPatchToolCall
from .beta_response_compaction_item_param import BetaResponseCompactionItemParam as BetaResponseCompactionItemParam
from .beta_response_custom_tool_call_item import BetaResponseCustomToolCallItem as BetaResponseCustomToolCallItem
from .beta_response_file_search_tool_call import BetaResponseFileSearchToolCall as BetaResponseFileSearchToolCall
from .beta_response_mcp_call_failed_event import BetaResponseMcpCallFailedEvent as BetaResponseMcpCallFailedEvent
from .assistant_tool_choice_function_param import AssistantToolChoiceFunctionParam as AssistantToolChoiceFunctionParam
from .beta_computer_use_preview_tool_param import BetaComputerUsePreviewToolParam as BetaComputerUsePreviewToolParam
from .beta_response_custom_tool_call_param import BetaResponseCustomToolCallParam as BetaResponseCustomToolCallParam
from .beta_response_output_item_done_event import BetaResponseOutputItemDoneEvent as BetaResponseOutputItemDoneEvent
from .beta_response_content_part_done_event import BetaResponseContentPartDoneEvent as BetaResponseContentPartDoneEvent
from .beta_response_custom_tool_call_output import BetaResponseCustomToolCallOutput as BetaResponseCustomToolCallOutput
from .beta_response_function_tool_call_item import BetaResponseFunctionToolCallItem as BetaResponseFunctionToolCallItem
from .beta_response_output_item_added_event import BetaResponseOutputItemAddedEvent as BetaResponseOutputItemAddedEvent
from .beta_response_tool_search_output_item import BetaResponseToolSearchOutputItem as BetaResponseToolSearchOutputItem
from .assistant_response_format_option_param import (
    AssistantResponseFormatOptionParam as AssistantResponseFormatOptionParam,
)
from .beta_container_network_policy_disabled import (
    BetaContainerNetworkPolicyDisabled as BetaContainerNetworkPolicyDisabled,
)
from .beta_response_computer_tool_call_param import (
    BetaResponseComputerToolCallParam as BetaResponseComputerToolCallParam,
)
from .beta_response_content_part_added_event import (
    BetaResponseContentPartAddedEvent as BetaResponseContentPartAddedEvent,
)
from .beta_response_conversation_param_param import (
    BetaResponseConversationParamParam as BetaResponseConversationParamParam,
)
from .beta_response_format_text_config_param import (
    BetaResponseFormatTextConfigParam as BetaResponseFormatTextConfigParam,
)
from .beta_response_function_shell_tool_call import (
    BetaResponseFunctionShellToolCall as BetaResponseFunctionShellToolCall,
)
from .beta_response_function_tool_call_param import (
    BetaResponseFunctionToolCallParam as BetaResponseFunctionToolCallParam,
)
from .beta_response_input_file_content_param import (
    BetaResponseInputFileContentParam as BetaResponseInputFileContentParam,
)
from .beta_response_input_text_content_param import (
    BetaResponseInputTextContentParam as BetaResponseInputTextContentParam,
)
from .beta_response_mcp_call_completed_event import (
    BetaResponseMcpCallCompletedEvent as BetaResponseMcpCallCompletedEvent,
)
from .beta_container_network_policy_allowlist import (
    BetaContainerNetworkPolicyAllowlist as BetaContainerNetworkPolicyAllowlist,
)
from .beta_response_function_call_output_item import (
    BetaResponseFunctionCallOutputItem as BetaResponseFunctionCallOutputItem,
)
from .beta_response_function_web_search_param import (
    BetaResponseFunctionWebSearchParam as BetaResponseFunctionWebSearchParam,
)
from .beta_response_input_image_content_param import (
    BetaResponseInputImageContentParam as BetaResponseInputImageContentParam,
)
from .beta_response_reasoning_text_done_event import (
    BetaResponseReasoningTextDoneEvent as BetaResponseReasoningTextDoneEvent,
)
from .beta_response_code_interpreter_tool_call import (
    BetaResponseCodeInterpreterToolCall as BetaResponseCodeInterpreterToolCall,
)
from .beta_response_input_message_content_list import (
    BetaResponseInputMessageContentList as BetaResponseInputMessageContentList,
)
from .beta_response_mcp_call_in_progress_event import (
    BetaResponseMcpCallInProgressEvent as BetaResponseMcpCallInProgressEvent,
)
from .beta_response_reasoning_text_delta_event import (
    BetaResponseReasoningTextDeltaEvent as BetaResponseReasoningTextDeltaEvent,
)
from .beta_response_audio_transcript_done_event import (
    BetaResponseAudioTranscriptDoneEvent as BetaResponseAudioTranscriptDoneEvent,
)
from .beta_response_compaction_item_param_param import (
    BetaResponseCompactionItemParamParam as BetaResponseCompactionItemParamParam,
)
from .beta_response_file_search_tool_call_param import (
    BetaResponseFileSearchToolCallParam as BetaResponseFileSearchToolCallParam,
)
from .beta_response_mcp_list_tools_failed_event import (
    BetaResponseMcpListToolsFailedEvent as BetaResponseMcpListToolsFailedEvent,
)
from .beta_response_apply_patch_tool_call_output import (
    BetaResponseApplyPatchToolCallOutput as BetaResponseApplyPatchToolCallOutput,
)
from .beta_response_audio_transcript_delta_event import (
    BetaResponseAudioTranscriptDeltaEvent as BetaResponseAudioTranscriptDeltaEvent,
)
from .beta_response_custom_tool_call_output_item import (
    BetaResponseCustomToolCallOutputItem as BetaResponseCustomToolCallOutputItem,
)
from .beta_container_network_policy_domain_secret import (
    BetaContainerNetworkPolicyDomainSecret as BetaContainerNetworkPolicyDomainSecret,
)
from .beta_response_custom_tool_call_output_param import (
    BetaResponseCustomToolCallOutputParam as BetaResponseCustomToolCallOutputParam,
)
from .beta_response_mcp_call_arguments_done_event import (
    BetaResponseMcpCallArgumentsDoneEvent as BetaResponseMcpCallArgumentsDoneEvent,
)
from .beta_response_tool_search_output_item_param import (
    BetaResponseToolSearchOutputItemParam as BetaResponseToolSearchOutputItemParam,
)
from .beta_container_network_policy_disabled_param import (
    BetaContainerNetworkPolicyDisabledParam as BetaContainerNetworkPolicyDisabledParam,
)
from .beta_response_computer_tool_call_output_item import (
    BetaResponseComputerToolCallOutputItem as BetaResponseComputerToolCallOutputItem,
)
from .beta_response_format_text_json_schema_config import (
    BetaResponseFormatTextJSONSchemaConfig as BetaResponseFormatTextJSONSchemaConfig,
)
from .beta_response_function_call_output_item_list import (
    BetaResponseFunctionCallOutputItemList as BetaResponseFunctionCallOutputItemList,
)
from .beta_response_function_tool_call_output_item import (
    BetaResponseFunctionToolCallOutputItem as BetaResponseFunctionToolCallOutputItem,
)
from .beta_response_image_gen_call_completed_event import (
    BetaResponseImageGenCallCompletedEvent as BetaResponseImageGenCallCompletedEvent,
)
from .beta_response_mcp_call_arguments_delta_event import (
    BetaResponseMcpCallArgumentsDeltaEvent as BetaResponseMcpCallArgumentsDeltaEvent,
)
from .beta_response_mcp_list_tools_completed_event import (
    BetaResponseMcpListToolsCompletedEvent as BetaResponseMcpListToolsCompletedEvent,
)
from .beta_container_network_policy_allowlist_param import (
    BetaContainerNetworkPolicyAllowlistParam as BetaContainerNetworkPolicyAllowlistParam,
)
from .beta_response_function_call_output_item_param import (
    BetaResponseFunctionCallOutputItemParam as BetaResponseFunctionCallOutputItemParam,
)
from .beta_response_function_shell_tool_call_output import (
    BetaResponseFunctionShellToolCallOutput as BetaResponseFunctionShellToolCallOutput,
)
from .beta_response_image_gen_call_generating_event import (
    BetaResponseImageGenCallGeneratingEvent as BetaResponseImageGenCallGeneratingEvent,
)
from .beta_response_web_search_call_completed_event import (
    BetaResponseWebSearchCallCompletedEvent as BetaResponseWebSearchCallCompletedEvent,
)
from .beta_response_web_search_call_searching_event import (
    BetaResponseWebSearchCallSearchingEvent as BetaResponseWebSearchCallSearchingEvent,
)
from .beta_response_code_interpreter_tool_call_param import (
    BetaResponseCodeInterpreterToolCallParam as BetaResponseCodeInterpreterToolCallParam,
)
from .beta_response_file_search_call_completed_event import (
    BetaResponseFileSearchCallCompletedEvent as BetaResponseFileSearchCallCompletedEvent,
)
from .beta_response_file_search_call_searching_event import (
    BetaResponseFileSearchCallSearchingEvent as BetaResponseFileSearchCallSearchingEvent,
)
from .beta_response_image_gen_call_in_progress_event import (
    BetaResponseImageGenCallInProgressEvent as BetaResponseImageGenCallInProgressEvent,
)
from .beta_response_input_message_content_list_param import (
    BetaResponseInputMessageContentListParam as BetaResponseInputMessageContentListParam,
)
from .beta_response_mcp_list_tools_in_progress_event import (
    BetaResponseMcpListToolsInProgressEvent as BetaResponseMcpListToolsInProgressEvent,
)
from .beta_response_custom_tool_call_input_done_event import (
    BetaResponseCustomToolCallInputDoneEvent as BetaResponseCustomToolCallInputDoneEvent,
)
from .beta_response_reasoning_summary_part_done_event import (
    BetaResponseReasoningSummaryPartDoneEvent as BetaResponseReasoningSummaryPartDoneEvent,
)
from .beta_response_reasoning_summary_text_done_event import (
    BetaResponseReasoningSummaryTextDoneEvent as BetaResponseReasoningSummaryTextDoneEvent,
)
from .beta_response_web_search_call_in_progress_event import (
    BetaResponseWebSearchCallInProgressEvent as BetaResponseWebSearchCallInProgressEvent,
)
from .beta_response_custom_tool_call_input_delta_event import (
    BetaResponseCustomToolCallInputDeltaEvent as BetaResponseCustomToolCallInputDeltaEvent,
)
from .beta_response_file_search_call_in_progress_event import (
    BetaResponseFileSearchCallInProgressEvent as BetaResponseFileSearchCallInProgressEvent,
)
from .beta_response_function_call_arguments_done_event import (
    BetaResponseFunctionCallArgumentsDoneEvent as BetaResponseFunctionCallArgumentsDoneEvent,
)
from .beta_response_function_shell_call_output_content import (
    BetaResponseFunctionShellCallOutputContent as BetaResponseFunctionShellCallOutputContent,
)
from .beta_response_image_gen_call_partial_image_event import (
    BetaResponseImageGenCallPartialImageEvent as BetaResponseImageGenCallPartialImageEvent,
)
from .beta_response_output_text_annotation_added_event import (
    BetaResponseOutputTextAnnotationAddedEvent as BetaResponseOutputTextAnnotationAddedEvent,
)
from .beta_response_reasoning_summary_part_added_event import (
    BetaResponseReasoningSummaryPartAddedEvent as BetaResponseReasoningSummaryPartAddedEvent,
)
from .beta_response_reasoning_summary_text_delta_event import (
    BetaResponseReasoningSummaryTextDeltaEvent as BetaResponseReasoningSummaryTextDeltaEvent,
)
from .beta_container_network_policy_domain_secret_param import (
    BetaContainerNetworkPolicyDomainSecretParam as BetaContainerNetworkPolicyDomainSecretParam,
)
from .beta_response_function_call_arguments_delta_event import (
    BetaResponseFunctionCallArgumentsDeltaEvent as BetaResponseFunctionCallArgumentsDeltaEvent,
)
from .beta_response_tool_search_output_item_param_param import (
    BetaResponseToolSearchOutputItemParamParam as BetaResponseToolSearchOutputItemParamParam,
)
from .beta_response_computer_tool_call_output_screenshot import (
    BetaResponseComputerToolCallOutputScreenshot as BetaResponseComputerToolCallOutputScreenshot,
)
from .beta_response_format_text_json_schema_config_param import (
    BetaResponseFormatTextJSONSchemaConfigParam as BetaResponseFormatTextJSONSchemaConfigParam,
)
from .beta_response_function_call_output_item_list_param import (
    BetaResponseFunctionCallOutputItemListParam as BetaResponseFunctionCallOutputItemListParam,
)
from .beta_response_code_interpreter_call_code_done_event import (
    BetaResponseCodeInterpreterCallCodeDoneEvent as BetaResponseCodeInterpreterCallCodeDoneEvent,
)
from .beta_response_code_interpreter_call_completed_event import (
    BetaResponseCodeInterpreterCallCompletedEvent as BetaResponseCodeInterpreterCallCompletedEvent,
)
from .beta_response_code_interpreter_call_code_delta_event import (
    BetaResponseCodeInterpreterCallCodeDeltaEvent as BetaResponseCodeInterpreterCallCodeDeltaEvent,
)
from .beta_response_code_interpreter_call_in_progress_event import (
    BetaResponseCodeInterpreterCallInProgressEvent as BetaResponseCodeInterpreterCallInProgressEvent,
)
from .beta_response_code_interpreter_call_interpreting_event import (
    BetaResponseCodeInterpreterCallInterpretingEvent as BetaResponseCodeInterpreterCallInterpretingEvent,
)
from .beta_response_function_shell_call_output_content_param import (
    BetaResponseFunctionShellCallOutputContentParam as BetaResponseFunctionShellCallOutputContentParam,
)
from .beta_response_computer_tool_call_output_screenshot_param import (
    BetaResponseComputerToolCallOutputScreenshotParam as BetaResponseComputerToolCallOutputScreenshotParam,
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Union
from typing_extensions import Required, TypedDict

__all__ = ["ChatSessionWorkflowParam", "Tracing"]


class Tracing(TypedDict, total=False):
    """Optional tracing overrides for the workflow invocation.

    When omitted, tracing is enabled by default.
    """

    enabled: bool
    """Whether tracing is enabled during the session. Defaults to true."""


class ChatSessionWorkflowParam(TypedDict, total=False):
    """Workflow reference and overrides applied to the chat session."""

    id: Required[str]
    """Identifier for the workflow invoked by the session."""

    state_variables: Dict[str, Union[str, bool, float]]
    """State variables forwarded to the workflow.

    Keys may be up to 64 characters, values must be primitive types, and the map
    defaults to an empty object.
    """

    tracing: Tracing
    """Optional tracing overrides for the workflow invocation.

    When omitted, tracing is enabled by default.
    """

    version: str
    """Specific workflow version to run. Defaults to the latest deployed version."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from .beta_response_custom_tool_call_output import BetaResponseCustomToolCallOutput

__all__ = ["BetaResponseCustomToolCallOutputItem"]


class BetaResponseCustomToolCallOutputItem(BetaResponseCustomToolCallOutput):
    """The output of a custom tool call from your code, being sent back to the model."""

    id: str  # type: ignore
    """The unique ID of the custom tool call output item."""

    status: Literal["in_progress", "completed", "incomplete"]
    """The status of the item.

    One of `in_progress`, `completed`, or `incomplete`. Populated when items are
    returned via API.
    """

    created_by: Optional[str] = None
    """The identifier of the actor that created the item."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .beta_response_input_item import BetaResponseInputItem

__all__ = ["BetaResponseInjectFailedEvent", "Error"]


class Error(BaseModel):
    """Information about why the input was not committed."""

    code: Literal["response_already_completed", "response_not_found"]
    """A machine-readable error code."""

    message: str
    """A human-readable description of the error."""


class BetaResponseInjectFailedEvent(BaseModel):
    """Emitted when injected input could not be committed to a response.

    The event
    returns the uncommitted raw input so the client can retry it in another
    response when appropriate.
    """

    error: Error
    """Information about why the input was not committed."""

    input: List[BetaResponseInputItem]
    """The raw input items that were not committed."""

    response_id: str
    """The ID of the response that rejected the input."""

    sequence_number: int
    """The sequence number for this event."""

    type: Literal["response.inject.failed"]
    """The event discriminator. Always `response.inject.failed`."""

    stream_id: Optional[str] = None
    """The multiplexed WebSocket stream that emitted the event.

    This field is present only when WebSocket multiplexing is enabled separately.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["TranscriptionDiarizedSegment"]


class TranscriptionDiarizedSegment(BaseModel):
    """A segment of diarized transcript text with speaker metadata."""

    id: str
    """Unique identifier for the segment."""

    end: float
    """End timestamp of the segment in seconds."""

    speaker: str
    """Speaker label for this segment.

    When known speakers are provided, the label matches `known_speaker_names[]`.
    Otherwise speakers are labeled sequentially using capital letters (`A`, `B`,
    ...).
    """

    start: float
    """Start timestamp of the segment in seconds."""

    text: str
    """Transcript text for this segment."""

    type: Literal["transcript.text.segment"]
    """The type of the segment. Always `transcript.text.segment`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Union, Optional
from typing_extensions import Literal, Annotated, TypeAlias

from ..._utils import PropertyInfo
from ..._models import BaseModel

__all__ = ["BetaResponseCustomToolCall", "Agent", "Caller", "CallerDirect", "CallerProgram"]


class Agent(BaseModel):
    """The agent that produced this item."""

    agent_name: str
    """The canonical name of the agent that produced this item."""


class CallerDirect(BaseModel):
    type: Literal["direct"]


class CallerProgram(BaseModel):
    caller_id: str
    """The call ID of the program item that produced this tool call."""

    type: Literal["program"]


Caller: TypeAlias = Annotated[Union[CallerDirect, CallerProgram, None], PropertyInfo(discriminator="type")]


class BetaResponseCustomToolCall(BaseModel):
    """A call to a custom tool created by the model."""

    call_id: str
    """An identifier used to map this custom tool call to a tool call output."""

    input: str
    """The input for the custom tool call generated by the model."""

    name: str
    """The name of the custom tool being called."""

    type: Literal["custom_tool_call"]
    """The type of the custom tool call. Always `custom_tool_call`."""

    id: Optional[str] = None
    """The unique ID of the custom tool call in the OpenAI platform."""

    agent: Optional[Agent] = None
    """The agent that produced this item."""

    caller: Optional[Caller] = None
    """The execution context that produced this tool call."""

    namespace: Optional[str] = None
    """The namespace of the custom tool being called."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Optional
from typing_extensions import Literal, Required, TypeAlias, TypedDict

__all__ = ["BetaCustomToolParam", "Format", "FormatText", "FormatGrammar"]


class FormatText(TypedDict, total=False):
    """Unconstrained free-form text."""

    type: Required[Literal["text"]]
    """Unconstrained text format. Always `text`."""


class FormatGrammar(TypedDict, total=False):
    """A grammar defined by the user."""

    definition: Required[str]
    """The grammar definition."""

    syntax: Required[Literal["lark", "regex"]]
    """The syntax of the grammar definition. One of `lark` or `regex`."""

    type: Required[Literal["grammar"]]
    """Grammar format. Always `grammar`."""


Format: TypeAlias = Union[FormatText, FormatGrammar]


class BetaCustomToolParam(TypedDict, total=False):
    """A custom tool that processes input using a specified format.

    Learn more about   [custom tools](https://platform.openai.com/docs/guides/function-calling#custom-tools)
    """

    name: Required[str]
    """The name of the custom tool, used to identify it in tool calls."""

    type: Required[Literal["custom"]]
    """The type of the custom tool. Always `custom`."""

    allowed_callers: Optional[List[Literal["direct", "programmatic"]]]
    """The tool invocation context(s)."""

    defer_loading: bool
    """Whether this tool should be deferred and discovered via tool search."""

    description: str
    """Optional description of the custom tool, used to provide more context."""

    format: Format
    """The input format for the custom tool. Default is unconstrained text."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .admin import (
    Admin,
    AsyncAdmin,
    AdminWithRawResponse,
    AsyncAdminWithRawResponse,
    AdminWithStreamingResponse,
    AsyncAdminWithStreamingResponse,
)
from .organization import (
    Organization,
    AsyncOrganization,
    OrganizationWithRawResponse,
    AsyncOrganizationWithRawResponse,
    OrganizationWithStreamingResponse,
    AsyncOrganizationWithStreamingResponse,
)

__all__ = [
    "Organization",
    "AsyncOrganization",
    "OrganizationWithRawResponse",
    "AsyncOrganizationWithRawResponse",
    "OrganizationWithStreamingResponse",
    "AsyncOrganizationWithStreamingResponse",
    "Admin",
    "AsyncAdmin",
    "AdminWithRawResponse",
    "AsyncAdminWithRawResponse",
    "AdminWithStreamingResponse",
    "AsyncAdminWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from .local_skill import LocalSkill

__all__ = ["LocalEnvironment"]


class LocalEnvironment(BaseModel):
    type: Literal["local"]
    """Use a local computer environment."""

    skills: Optional[List[LocalSkill]] = None
    """An optional list of skills."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["InputAudioBufferDtmfEventReceivedEvent"]


class InputAudioBufferDtmfEventReceivedEvent(BaseModel):
    """**SIP Only:** Returned when an DTMF event is received.

    A DTMF event is a message that
    represents a telephone keypad press (0–9, *, #, A–D). The `event` property
    is the keypad that the user press. The `received_at` is the UTC Unix Timestamp
    that the server received the event.
    """

    event: str
    """The telephone keypad that was pressed by the user."""

    received_at: int
    """UTC Unix Timestamp when DTMF Event was received by server."""

    type: Literal["input_audio_buffer.dtmf_event_received"]
    """The event type, must be `input_audio_buffer.dtmf_event_received`."""



from __future__ import annotations

import os
import re
import inspect
from typing import Literal, Callable, Awaitable, cast
from dataclasses import field, dataclass

import httpx

from .._types import NOT_GIVEN, NotGiven
from .._utils import asyncify
from .._httpx2 import normalize_httpx_url, request_not_read_exceptions
from .._models import FinalRequestOptions
from .._provider import _Provider, _create_provider, _ProviderRuntime
from .._exceptions import OpenAIError
from ..lib._bedrock_auth import (
    BedrockAwsAuth,
    BedrockAwsAuthConfig,
    AwsCredentialsProvider,
)

BedrockTokenProvider = Callable[[], "str | Awaitable[str]"]

_AWS_SIGNING_HEADERS = ("authorization", "x-amz-content-sha256", "x-amz-date", "x-amz-security-token")
_CANONICAL_BEDROCK_HOST = re.compile(r"^bedrock-mantle\.([a-z0-9-]+)\.api\.aws$", re.IGNORECASE)


def _normalize_optional_string(value: str | None) -> str | None:
    if value is None:
        return None

    normalized = value.strip()
    return normalized or None


def _normalize_base_url(base_url: str | httpx.URL) -> httpx.URL:
    url = normalize_httpx_url(base_url)
    path = url.path.rstrip("/")
    responses_match = re.search(r"/responses(?:/.*)?$", path)
    if responses_match is not None:
        path = path[: responses_match.start()]

    return url.copy_with(path=path or "/")


def _same_origin(left: httpx.URL, right: httpx.URL) -> bool:
    return (left.scheme, left.host, left.port) == (right.scheme, right.host, right.port)


def _body_for_signing(request: httpx.Request) -> bytes:
    try:
        return request.content
    except request_not_read_exceptions() as exc:
        raise OpenAIError(
            "Bedrock SigV4 authentication requires a replayable request body. "
            "Buffer the body before sending or use bearer authentication."
        ) from exc


def _assert_provider_owns_authorization(request: httpx.Request) -> None:
    if "Authorization" in request.headers:
        raise OpenAIError("Bedrock provider authentication cannot be combined with a custom `Authorization` header.")


def _without_redirects(options: FinalRequestOptions) -> FinalRequestOptions:
    if options.follow_redirects:
        raise OpenAIError(
            "Bedrock SigV4 authentication does not support automatic redirects. "
            "Send a new request to the redirect target so it can be signed again."
        )
    options.follow_redirects = False
    return options


class _BedrockBearerAuth:
    def __init__(self, token_provider: BedrockTokenProvider, *, base_url: httpx.URL) -> None:
        self._token_provider = token_provider
        self._base_url = base_url

    def _validate_request(self, request: httpx.Request) -> None:
        _assert_provider_owns_authorization(request)
        if not _same_origin(request.url, self._base_url):
            raise OpenAIError(
                "Refusing to authenticate a Bedrock request for an origin other than the configured provider URL."
            )

    def _resolve_token(self) -> str:
        try:
            token = cast(object, self._token_provider())
        except OpenAIError:
            raise
        except Exception as exc:
            raise OpenAIError("Failed to resolve a bearer credential for Bedrock.") from exc

        if inspect.isawaitable(token):
            close = getattr(token, "close", None)
            if callable(close):
                close()
            raise OpenAIError("An async Bedrock token provider requires `AsyncOpenAI`.")
        if not isinstance(token, str) or not token.strip():
            raise OpenAIError("The Bedrock bearer credential provider must return a non-empty string.")
        return token

    async def _resolve_token_async(self) -> str:
        try:
            token = cast(object, self._token_provider())
            if inspect.isawaitable(token):
                token = await token
        except OpenAIError:
            raise
        except Exception as exc:
            raise OpenAIError("Failed to resolve a bearer credential for Bedrock.") from exc

        if not isinstance(token, str) or not token.strip():
            raise OpenAIError("The Bedrock bearer credential provider must return a non-empty string.")
        return token

    def prepare_request(self, request: httpx.Request) -> None:
        self._validate_request(request)
        request.headers["Authorization"] = f"Bearer {self._resolve_token()}"

    async def prepare_async_request(self, request: httpx.Request) -> None:
        self._validate_request(request)
        request.headers["Authorization"] = f"Bearer {await self._resolve_token_async()}"


class _BedrockSigV4Auth:
    def __init__(
        self,
        *,
        config: BedrockAwsAuthConfig,
        base_url: httpx.URL,
        auth: BedrockAwsAuth | None = None,
    ) -> None:
        self._config = config
        self._base_url = base_url
        self._auth = auth

    def _validate_request(self, request: httpx.Request) -> bytes:
        _assert_provider_owns_authorization(request)
        if not _same_origin(request.url, self._base_url):
            raise OpenAIError(
                "Refusing to sign a Bedrock request for an origin other than the configured provider URL."
            )

        endpoint_region_match = _CANONICAL_BEDROCK_HOST.fullmatch(request.url.host)
        if endpoint_region_match is not None and endpoint_region_match.group(1) != self._config.region:
            raise OpenAIError(
                f"The Bedrock endpoint region `{endpoint_region_match.group(1)}` does not match the "
                f"SigV4 region `{self._config.region}`."
            )

        return _body_for_signing(request)

    def _sign(self, request: httpx.Request, *, auth: BedrockAwsAuth, body: bytes) -> None:
        for header in _AWS_SIGNING_HEADERS:
            request.headers.pop(header, None)

        signed_headers = auth.sign(
            method=request.method,
            url=str(request.url),
            headers=dict(request.headers),
            body=body,
        )
        request.headers.clear()
        request.headers.update(signed_headers)

    def prepare_request(self, request: httpx.Request) -> None:
        body = self._validate_request(request)
        if self._auth is None:
            self._auth = BedrockAwsAuth(self._config)
        self._sign(request, auth=self._auth, body=body)

    async def prepare_async_request(self, request: httpx.Request) -> None:
        body = self._validate_request(request)
        if self._auth is None:
            self._auth = await asyncify(BedrockAwsAuth)(self._config)

        signed_headers = await asyncify(self._auth.sign)(
            method=request.method,
            url=str(request.url),
            headers={
                name: value for name, value in request.headers.items() if name.lower() not in _AWS_SIGNING_HEADERS
            },
            body=body,
        )
        request.headers.clear()
        request.headers.update(signed_headers)


@dataclass
class _BedrockProviderRuntime(_ProviderRuntime):
    region: str | None = None


@dataclass(frozen=True)
class _BedrockProviderDefinition:
    configured_region: str | None
    region_source: Literal["explicit", "environment"] | None
    configured_base_url: httpx.URL | None
    api_key: str | None = field(default=None, repr=False)
    token_provider: BedrockTokenProvider | None = field(default=None, repr=False, compare=False)
    use_environment_bearer: bool = False
    profile: str | None = None
    access_key_id: str | None = field(default=None, repr=False)
    secret_access_key: str | None = field(default=None, repr=False)
    session_token: str | None = field(default=None, repr=False)
    credential_provider: AwsCredentialsProvider | None = field(default=None, repr=False, compare=False)
    name: str = field(default="bedrock", init=False)

    def _aws_source(self) -> Literal["static", "profile", "provider", "default"]:
        if self.access_key_id is not None:
            return "static"
        if self.profile is not None:
            return "profile"
        if self.credential_provider is not None:
            return "provider"
        return "default"

    def _resolve_aws_auth(self) -> tuple[BedrockAwsAuthConfig, BedrockAwsAuth | None]:
        if self.configured_region is not None:
            return (
                BedrockAwsAuthConfig(
                    region=self.configured_region,
                    source=self._aws_source(),
                    region_source=self.region_source or "explicit",
                    profile=self.profile,
                    access_key_id=self.access_key_id,
                    secret_access_key=self.secret_access_key,
                    session_token=self.session_token,
                    credentials_provider=self.credential_provider,
                ),
                None,
            )

        auth = BedrockAwsAuth.resolve(
            region=None,
            profile=self.profile,
            access_key_id=self.access_key_id,
            secret_access_key=self.secret_access_key,
            session_token=self.session_token,
            credentials_provider=self.credential_provider,
        )
        return auth.config, auth

    def configure(self) -> _ProviderRuntime:
        def environment_token() -> str:
            token = os.environ.get("AWS_BEARER_TOKEN_BEDROCK")
            if not token:
                raise OpenAIError(
                    "Could not find credentials for Bedrock. Pass a bearer credential or AWS credentials to "
                    "`bedrock(...)`, set `AWS_BEARER_TOKEN_BEDROCK`, or configure the default AWS credential chain."
                )
            return token

        auth: _BedrockBearerAuth | _BedrockSigV4Auth | None = None
        bearer_provider: BedrockTokenProvider | None = None
        if self.api_key is not None:
            bearer_provider = lambda: self.api_key or ""
            region = self.configured_region
        elif self.token_provider is not None:
            bearer_provider = self.token_provider
            region = self.configured_region
        elif self.use_environment_bearer:
            bearer_provider = environment_token
            region = self.configured_region
        else:
            aws_config, aws_auth = self._resolve_aws_auth()
            region = aws_config.region
            base_url = self.configured_base_url or _normalize_base_url(
                f"https://bedrock-mantle.{region}.api.aws/openai/v1"
            )
            auth = _BedrockSigV4Auth(config=aws_config, base_url=base_url, auth=aws_auth)

        if self.configured_base_url is not None:
            base_url = self.configured_base_url
        elif region is not None:
            base_url = _normalize_base_url(f"https://bedrock-mantle.{region}.api.aws/openai/v1")
        else:
            raise OpenAIError(
                "Bedrock requires an AWS region. Pass `region` to `bedrock(...)`, or set `AWS_REGION` or "
                "`AWS_DEFAULT_REGION`."
            )

        if bearer_provider is not None:
            auth = _BedrockBearerAuth(bearer_provider, base_url=base_url)

        assert auth is not None
        if isinstance(auth, _BedrockSigV4Auth):
            return _BedrockProviderRuntime(
                name=self.name,
                base_url=base_url,
                region=region,
                transform_request=_without_redirects,
                prepare_request=auth.prepare_request,
                prepare_async_request=auth.prepare_async_request,
            )

        return _BedrockProviderRuntime(
            name=self.name,
            base_url=base_url,
            region=region,
            prepare_request=auth.prepare_request,
            prepare_async_request=auth.prepare_async_request,
        )


def bedrock(
    *,
    region: str | None = None,
    base_url: str | httpx.URL | None | NotGiven = NOT_GIVEN,
    api_key: str | None | NotGiven = NOT_GIVEN,
    token_provider: BedrockTokenProvider | None = None,
    access_key_id: str | None = None,
    secret_access_key: str | None = None,
    session_token: str | None = None,
    profile: str | None = None,
    credential_provider: AwsCredentialsProvider | None = None,
) -> _Provider:
    """Configure the standard OpenAI client for Amazon Bedrock Mantle."""

    normalized_region = _normalize_optional_string(region)
    if region is not None and normalized_region is None:
        raise OpenAIError("The Bedrock AWS `region` must not be empty.")

    region_source: Literal["explicit", "environment"] | None = None
    if normalized_region is not None:
        region_source = "explicit"
    else:
        normalized_region = _normalize_optional_string(
            os.environ.get("AWS_REGION") or os.environ.get("AWS_DEFAULT_REGION")
        )
        if normalized_region is not None:
            region_source = "environment"

    configured_base_url: httpx.URL | None
    if isinstance(base_url, NotGiven):
        environment_base_url = _normalize_optional_string(os.environ.get("AWS_BEDROCK_BASE_URL"))
        configured_base_url = _normalize_base_url(environment_base_url) if environment_base_url else None
    elif base_url is None:
        configured_base_url = None
    else:
        if isinstance(base_url, str) and not base_url.strip():
            raise OpenAIError("The Bedrock `base_url` must not be empty.")
        configured_base_url = _normalize_base_url(base_url)

    normalized_profile = _normalize_optional_string(profile)
    if profile is not None and normalized_profile is None:
        raise OpenAIError("The Bedrock AWS `profile` must not be empty.")

    if (access_key_id is None) != (secret_access_key is None) or (session_token is not None and access_key_id is None):
        raise OpenAIError(
            "Static AWS credentials require both `access_key_id` and `secret_access_key`. "
            "A `session_token` may only be used with both."
        )
    if access_key_id is not None and (not access_key_id.strip() or not cast(str, secret_access_key).strip()):
        raise OpenAIError("Static AWS credentials require non-empty `access_key_id` and `secret_access_key` values.")
    if session_token is not None and not session_token.strip():
        raise OpenAIError("A static AWS `session_token` must not be empty when provided.")

    explicit_api_key = not isinstance(api_key, NotGiven) and api_key is not None
    if explicit_api_key and (not isinstance(api_key, str) or not api_key.strip()):
        raise OpenAIError("The Bedrock bearer credential must not be empty.")
    if explicit_api_key and token_provider is not None:
        raise OpenAIError("The `api_key` and `token_provider` options are mutually exclusive. Configure only one.")

    explicit_bearer = explicit_api_key or token_provider is not None
    aws_modes = sum(
        (
            access_key_id is not None,
            normalized_profile is not None,
            credential_provider is not None,
        )
    )
    if aws_modes > 1:
        raise OpenAIError(
            "Bedrock authentication is ambiguous. Configure exactly one explicit AWS mode: static credentials, "
            "profile, or credential provider."
        )
    if explicit_bearer and aws_modes:
        raise OpenAIError(
            "Bedrock authentication is ambiguous. Configure exactly one explicit mode: bearer credential, "
            "static AWS credentials, profile, or credential provider."
        )

    skip_environment_bearer = not isinstance(api_key, NotGiven) and api_key is None
    use_environment_bearer = (
        not explicit_bearer
        and not aws_modes
        and not skip_environment_bearer
        and bool(os.environ.get("AWS_BEARER_TOKEN_BEDROCK"))
    )

    return _create_provider(
        _BedrockProviderDefinition(
            configured_region=normalized_region,
            region_source=region_source,
            configured_base_url=configured_base_url,
            api_key=cast("str | None", api_key) if explicit_api_key else None,
            token_provider=token_provider,
            use_environment_bearer=use_environment_bearer,
            profile=normalized_profile,
            access_key_id=access_key_id,
            secret_access_key=secret_access_key,
            session_token=session_token,
            credential_provider=credential_provider,
        )
    )


__all__ = ["bedrock", "BedrockTokenProvider", "AwsCredentialsProvider"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["ProjectRateLimit"]


class ProjectRateLimit(BaseModel):
    """Represents a project rate limit config."""

    id: str
    """The identifier, which can be referenced in API endpoints."""

    max_requests_per_1_minute: int
    """The maximum requests per minute."""

    max_tokens_per_1_minute: int
    """The maximum tokens per minute."""

    model: str
    """The model this rate limit applies to."""

    object: Literal["project.rate_limit"]
    """The object type, which is always `project.rate_limit`"""

    batch_1_day_max_input_tokens: Optional[int] = None
    """The maximum batch input tokens per day. Only present for relevant models."""

    max_audio_megabytes_per_1_minute: Optional[int] = None
    """The maximum audio megabytes per minute. Only present for relevant models."""

    max_images_per_1_minute: Optional[int] = None
    """The maximum images per minute. Only present for relevant models."""

    max_requests_per_1_day: Optional[int] = None
    """The maximum requests per day. Only present for relevant models."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal

from ..._models import BaseModel
from ..shared.metadata import Metadata
from .conversation_item import ConversationItem
from .realtime_audio_formats import RealtimeAudioFormats
from .realtime_response_usage import RealtimeResponseUsage
from .realtime_response_status import RealtimeResponseStatus

__all__ = ["RealtimeResponse", "Audio", "AudioOutput"]


class AudioOutput(BaseModel):
    format: Optional[RealtimeAudioFormats] = None
    """The format of the output audio."""

    voice: Union[
        str, Literal["alloy", "ash", "ballad", "coral", "echo", "sage", "shimmer", "verse", "marin", "cedar"], None
    ] = None
    """The voice the model uses to respond.

    Voice cannot be changed during the session once the model has responded with
    audio at least once. Current voice options are `alloy`, `ash`, `ballad`,
    `coral`, `echo`, `sage`, `shimmer`, `verse`, `marin`, and `cedar`. We recommend
    `marin` and `cedar` for best quality.
    """


class Audio(BaseModel):
    """Configuration for audio output."""

    output: Optional[AudioOutput] = None


class RealtimeResponse(BaseModel):
    """The response resource."""

    id: Optional[str] = None
    """The unique ID of the response, will look like `resp_1234`."""

    audio: Optional[Audio] = None
    """Configuration for audio output."""

    conversation_id: Optional[str] = None
    """
    Which conversation the response is added to, determined by the `conversation`
    field in the `response.create` event. If `auto`, the response will be added to
    the default conversation and the value of `conversation_id` will be an id like
    `conv_1234`. If `none`, the response will not be added to any conversation and
    the value of `conversation_id` will be `null`. If responses are being triggered
    automatically by VAD the response will be added to the default conversation
    """

    max_output_tokens: Union[int, Literal["inf"], None] = None
    """
    Maximum number of output tokens for a single assistant response, inclusive of
    tool calls, that was used in this response.
    """

    metadata: Optional[Metadata] = None
    """Set of 16 key-value pairs that can be attached to an object.

    This can be useful for storing additional information about the object in a
    structured format, and querying for objects via API or the dashboard.

    Keys are strings with a maximum length of 64 characters. Values are strings with
    a maximum length of 512 characters.
    """

    object: Optional[Literal["realtime.response"]] = None
    """The object type, must be `realtime.response`."""

    output: Optional[List[ConversationItem]] = None
    """The list of output items generated by the response."""

    output_modalities: Optional[List[Literal["text", "audio"]]] = None
    """
    The set of modalities the model used to respond, currently the only possible
    values are `[\"audio\"]`, `[\"text\"]`. Audio output always include a text
    transcript. Setting the output to mode `text` will disable audio output from the
    model.
    """

    status: Optional[Literal["completed", "cancelled", "failed", "incomplete", "in_progress"]] = None
    """
    The final status of the response (`completed`, `cancelled`, `failed`, or
    `incomplete`, `in_progress`).
    """

    status_details: Optional[RealtimeResponseStatus] = None
    """Additional details about the status."""

    usage: Optional[RealtimeResponseUsage] = None
    """Usage statistics for the Response, this will correspond to billing.

    A Realtime API session will maintain a conversation context and append new Items
    to the Conversation, thus output from previous turns (text and audio tokens)
    will become the input for later turns.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["UserRetrieveResponse"]


class UserRetrieveResponse(BaseModel):
    """Details about a user returned from an organization group membership lookup."""

    id: str
    """Identifier for the user."""

    email: Optional[str] = None
    """Email address of the user, or `null` for users without an email."""

    is_service_account: Optional[bool] = None
    """Whether the user is a service account."""

    name: str
    """Display name of the user."""

    picture: Optional[str] = None
    """URL of the user's profile picture, if available."""

    user_type: Literal["user", "tenant_user"]
    """The type of user."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..responses.response_input_file import ResponseInputFile

__all__ = ["InputFileContent"]

InputFileContent = ResponseInputFile



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["GroupUpdateResponse"]


class GroupUpdateResponse(BaseModel):
    """Response returned after updating a group."""

    id: str
    """Identifier for the group."""

    created_at: int
    """Unix timestamp (in seconds) when the group was created."""

    is_scim_managed: bool
    """
    Whether the group is managed through SCIM and controlled by your identity
    provider.
    """

    name: str
    """Updated display name for the group."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ......_models import BaseModel

__all__ = ["RoleDeleteResponse"]


class RoleDeleteResponse(BaseModel):
    """Confirmation payload returned after unassigning a role."""

    deleted: bool
    """Whether the assignment was removed."""

    object: str
    """
    Identifier for the deleted assignment, such as `group.role.deleted` or
    `user.role.deleted`.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ..responses.response_input_text import ResponseInputText

__all__ = ["InputTextContent"]

InputTextContent = ResponseInputText



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal

from ..._models import BaseModel

__all__ = ["AudioTranscription"]


class AudioTranscription(BaseModel):
    delay: Optional[Literal["minimal", "low", "medium", "high", "xhigh"]] = None
    """
    Controls how long the model waits before emitting transcription text. Higher
    values can improve transcription accuracy at the cost of latency. Only supported
    with `gpt-realtime-whisper` in GA Realtime sessions.
    """

    keywords: Optional[List[str]] = None
    """Words or phrases to guide transcription of the input audio.

    Supported by `gpt-transcribe` and `gpt-live-transcribe`.
    """

    language: Optional[str] = None
    """The language of the input audio.

    Supplying the input language in
    [ISO-639-1](https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes) (e.g. `en`)
    format will improve accuracy and latency.
    """

    languages: Optional[List[str]] = None
    """
    Possible languages of the input audio, in
    [ISO-639-1](https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes) format.
    Supported by `gpt-transcribe` and `gpt-live-transcribe`.
    """

    model: Union[
        str,
        Literal[
            "whisper-1",
            "gpt-transcribe",
            "gpt-live-transcribe",
            "gpt-4o-mini-transcribe",
            "gpt-4o-mini-transcribe-2025-12-15",
            "gpt-4o-transcribe",
            "gpt-4o-transcribe-diarize",
            "gpt-realtime-whisper",
        ],
        None,
    ] = None
    """The model to use for transcription.

    Current options are `whisper-1`, `gpt-transcribe`, `gpt-live-transcribe`,
    `gpt-4o-mini-transcribe`, `gpt-4o-mini-transcribe-2025-12-15`,
    `gpt-4o-transcribe`, `gpt-4o-transcribe-diarize`, and `gpt-realtime-whisper`.
    Use `gpt-4o-transcribe-diarize` when you need diarization with speaker labels.
    """

    prompt: Optional[str] = None
    """
    An optional text to guide the model's style or continue a previous audio
    segment. For `whisper-1`, the
    [prompt is a list of keywords](https://platform.openai.com/docs/guides/speech-to-text#prompting).
    For `gpt-4o-transcribe` models (excluding `gpt-4o-transcribe-diarize`), the
    prompt is a free text string, for example "expect words related to technology".
    Prompt is not supported with `gpt-realtime-whisper` in GA Realtime sessions.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal, Required, TypedDict

__all__ = ["BetaContainerNetworkPolicyDisabledParam"]


class BetaContainerNetworkPolicyDisabledParam(TypedDict, total=False):
    type: Required[Literal["disabled"]]
    """Disable outbound network access. Always `disabled`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ...._models import BaseModel

__all__ = ["ChatSessionFileUpload"]


class ChatSessionFileUpload(BaseModel):
    """Upload permissions and limits applied to the session."""

    enabled: bool
    """Indicates if uploads are enabled for the session."""

    max_file_size: Optional[int] = None
    """Maximum upload size in megabytes."""

    max_files: Optional[int] = None
    """Maximum number of uploads allowed during the session."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List
from typing_extensions import TypeAlias

from .beta_response_function_call_output_item import BetaResponseFunctionCallOutputItem

__all__ = ["BetaResponseFunctionCallOutputItemList"]

BetaResponseFunctionCallOutputItemList: TypeAlias = List[BetaResponseFunctionCallOutputItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["RateLimitUpdateRateLimitParams"]


class RateLimitUpdateRateLimitParams(TypedDict, total=False):
    project_id: Required[str]

    batch_1_day_max_input_tokens: int
    """The maximum batch input tokens per day. Only relevant for certain models."""

    max_audio_megabytes_per_1_minute: int
    """The maximum audio megabytes per minute. Only relevant for certain models."""

    max_images_per_1_minute: int
    """The maximum images per minute. Only relevant for certain models."""

    max_requests_per_1_day: int
    """The maximum requests per day. Only relevant for certain models."""

    max_requests_per_1_minute: int
    """The maximum requests per minute."""

    max_tokens_per_1_minute: int
    """The maximum tokens per minute."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional
from typing_extensions import Literal

from ....._models import BaseModel

__all__ = ["ProjectAPIKey", "Owner", "OwnerServiceAccount", "OwnerUser"]


class OwnerServiceAccount(BaseModel):
    """The service account that owns a project API key."""

    id: str
    """The identifier, which can be referenced in API endpoints"""

    created_at: int
    """The Unix timestamp (in seconds) of when the service account was created."""

    name: str
    """The name of the service account."""

    role: str
    """The service account's project role."""


class OwnerUser(BaseModel):
    """The user that owns a project API key."""

    id: str
    """The identifier, which can be referenced in API endpoints"""

    created_at: int
    """The Unix timestamp (in seconds) of when the user was created."""

    email: str
    """The email address of the user."""

    name: str
    """The name of the user."""

    role: str
    """The user's project role."""


class Owner(BaseModel):
    service_account: Optional[OwnerServiceAccount] = None
    """The service account that owns a project API key."""

    type: Optional[Literal["user", "service_account"]] = None
    """`user` or `service_account`"""

    user: Optional[OwnerUser] = None
    """The user that owns a project API key."""


class ProjectAPIKey(BaseModel):
    """Represents an individual API key in a project."""

    id: str
    """The identifier, which can be referenced in API endpoints"""

    created_at: int
    """The Unix timestamp (in seconds) of when the API key was created"""

    last_used_at: Optional[int] = None
    """The Unix timestamp (in seconds) of when the API key was last used."""

    name: str
    """The name of the API key"""

    object: Literal["organization.project.api_key"]
    """The object type, which is always `organization.project.api_key`"""

    owner: Owner

    owner_project_access: Literal["active", "inactive"]
    """Whether the API key's owner currently has effective access to the project."""

    redacted_value: str
    """The redacted value of the API key"""
